// This file is part of QuickPlot

/*********************************************************************
 * Class PLT_Class_Data
 *
 * Public function:
 *     PLT_Class_Data ... Class method interface
 *
 * Properties:
 *     name     ... name of data
 *     nRow     ... the number of rows of numeric data
 *     nCol     ... the number of columns of numeric data
 *     varNames ... string array of variable names
 *     numeric  ... numeric data
 *
 * Methods:
 *     new            ... Create a new instance
 *     show           ... Display data
 *     plot           ... Plot a part of data
 *     importNumeric  ... Import numeric data from ascii file
 *     importVarNames ... Import variable names from ascii file
 ********************************************************************/

//--------------------------------------------------------------------
function Data = PLT_Class_Data()
    
    // Create a Data class
    
    Data = PLT_Util_createTList('PLT_DataClass', struct(..
        'new', Data_new,..
        'show', Data_show,..
        'plot', Data_plot,..
        'importNumeric', Data_importNumeric,..
        'importVarNames', Data_importVarNames..
    ));
    
endfunction

//--------------------------------------------------------------------
function this = Data_new()

    // Create a new data
    
    this = PLT_Util_createTList('PLT_Data', struct(..
        'name', '',..
        'nRow', 0,..
        'nCol', 0,..
        'varNames', [],..
        'numeric', []..
    ));

endfunction

//--------------------------------------------------------------------
function Data_show(this)

    // Display data
    
    mprintf('PLT_Data:\n');
    mprintf('===================================\n');
    for i = 2:length(this)
        mprintf('%s:', this(1)(i));
        disp(this(i));
    end

endfunction

//--------------------------------------------------------------------
function polylineHandles = Data_plot(this, xid, yids, varargin)
    
    // Plot a part of data
    //
    //     xid ... column index of x variable
    //     yids ... column indices of y variables
    //     varargin ... [ axesHandle ]
    //         axesHandle ... handle of axes in which data is plotted
    //
    //     polylineHandles ... handles of the plot polylines unglued under the axes

    // if axesHandle is not provided, create new axes.
    nArgIn = argn(2);
    if nArgIn >= 1 then
        axesHandle = varargin(1);
    else
        axesHandle = gca();
    end
    
    axesHandle = sca(axesHandle);
    y = this.numeric(:, yids);
    if isempty(xid) then
        plot(y);
    else
        x = this.numeric(:, xid);
        plot(x, y);
    end
    polylineHandles = unglue(gce());
    
endfunction

//--------------------------------------------------------------------
function this = Data_importNumeric(this, filename, varargin)

    // Import Data from ascii file
    //
    //     filename ... file path name
    //     varargin ... [ skiprows, sep ]
    //         skiprows ... number of rows skipped (if not set, 0)
    //         sep      ... field separator (if not set, space & tab)
    
    // default parameter values
    skiprows = 0;
    sep = [' ', ascii(9)]; // ascii(9)=<TAB>
    
    // process varargin
    nArgIn = argn(2);
    if nArgIn >= 1 then
        skiprows = varargin(1);
    end
    if nArgIn >= 2 then
        sep = varargin(2);
    end
    
    if size(sep, 1) == 1 && size(sep, 2) == 1 then
        M = csvRead(filename, sep, '.', 'double', [], [], [], skiprows);
    else
        // open file
        fid = mopen(filename, 'r');
        // skip rows
        mgetl(fid, skiprows);
        // read the rest rows
        Mstr = mgetl(fid);
        // close file
        mclose(fid);
        // initialize
        M = [];
        nc = 0;
        // read data line by line
        for i = 1:size(Mstr, 'r')
            line = Mstr(i);
            // skip if encountered with a blank line
            if isempty(line) then
                continue;
            end
            linedata = strtod(tokens(line, sep)).';
            if i == 1 then
                nc = length(linedata); // the number of columns
            else
                // error if encountered with an inconsistent data
                if length(linedata) ~= nc then
                    msg = msprintf("There is inconsistent data at line %d in file %s.\nData read is aborted.\n", i+skiprows, filename);
                    error(msg);
                end
            end
            M($+1,:) = linedata;
        end
    end
    this.nRow = size(M, 'r');
    this.nCol = size(M, 'c');
    this.numeric = M;

endfunction

//--------------------------------------------------------------------
function this = Data_importVarNames(this, filename, row, varargin)
    
    // Import variable names from ascii file
    //
    //     filename ... file path name
    //     row ... row in the file where the label is
    //     varargin ... [ sep ]
    //         sep ... field separator (if not set, space & tab)

    // default parameter values
    sep = [' ', ascii(9)]; // ascii(9)=<TAB>
    
    // process varargin
    nArgIn = argn(2);
    if nArgIn >= 1 then
        sep = varargin(1);
    end
    
    // import variable names
    fid = mopen(filename, 'r');
    line = mgetl(fid, row);
    mclose(fid);
    varNames = stripblanks(tokens(line($), sep));
    
    // if there is a duplicated name, put a unique subscript to the name.
    for i = 2:1:prod(size(varNames))
        if ~isempty(find(varNames(1:i-1) == varNames(i)))
            sub = 1; 
            while ~isempty(find(varNames(1:i-1) == (varNames(i) + '_' + string(sub))))
                sub = sub + 1;
            end
            varNames(i) = varNames(i) + '_' + string(sub);
        end
    end

    if size(varNames, 'r') ~= this.nCol then
        msg = msprintf("The number of variable names must match the number of columns of data (%d).", this.nCol);
        error(msg);
    end
    
    this.varNames = varNames;

endfunction
