// This file is part of QuickPlot

/*********************************************************************
 * Class PLT_Class_DataList
 *
 * Public function:
 *     PLT_Class_DataList ... Class method interface
 *
 * Properties:
 *     name ... name of data list
 *     data ... list of data
 *
 * Methods:
 *     new         ... Create a new instance
 *     show        ... Display data list
 *     addData     ... Add a data to a data list
 *     delData     ... Delete a data from a data list
 *     permData    ... Change the order of data in a data list
 *     replaceData ... Replace a data in a data list
 ********************************************************************/

//--------------------------------------------------------------------
function DataList = PLT_Class_DataList()
    
    // Create a DataList class
    
    DataList = PLT_Util_createTList('PLT_DataListClass', struct(..
        'new', DataList_new,..
        'show', DataList_show,..
        'addData', DataList_addData,..
        'delData', DataList_delData,..
        'permData', DataList_permData,..
        'replaceData', DataList_replaceData..
    ));
    
endfunction

//--------------------------------------------------------------------
function this = DataList_new()

    // Create a new data list.
    
    this = PLT_Util_createTList('PLT_DataList', struct(..
        'name', '',..
        'data', list()..
    ));

endfunction

//--------------------------------------------------------------------
function DataList_show(this)

    // Display data list.
    
    mprintf('PLT_DataList:\n');
    mprintf('===================================\n');
    mprintf('name: %s\n', this.name);
    mprintf('data: (List of Data)\n');
    if isempty(this.data)
        mprintf('    -- empty --\n');
    else
        for i = 1:length(this.data)
            mprintf('    Data(%d): ""%s""\n',...
                     i, this.data(i).name);
        end
    end

endfunction

//--------------------------------------------------------------------
function this = DataList_addData(this, data)

    // Add a data to a data list

    this.data($+1) = data;

endfunction

//--------------------------------------------------------------------
function this = DataList_delData(this, index)

    // Delete a data from a data list

    this.data(index) = null();

endfunction

//--------------------------------------------------------------------
function this = DataList_permData(this, fromIndex, toIndex)

    // Change the order of data in a data list.
    //
    // The data at the fromIndex position is moved to the toIndex position
    // and the other data are shifted orderly.

    tmp = this.data(fromIndex);
    this.data(fromIndex) = null();
    for i = length(this.data) : -1 : toIndex
        this.data(i+1) = this.data(i);
    end
    this.data(toIndex) = tmp;

endfunction

//--------------------------------------------------------------------
function this = DataList_replaceData(this, index, newData)
    
    // Replace a data in a data list.
    //
    // The data at the index position is replaced with newData.

    this.data(index) = newData;
    
endfunction
