// This file is part of QuickPlot

//function polylineHandles = PLT_Macro_plotData(dataIndices, xVarIndex, yVarIndices, newPlotYesNo)
function PLT_Macro_plotData(dataIndices, xVarIndex, yVarIndices, newPlotYesNo)
    
    /****************************************************************
     * Plot data and add legend
     *
     * Input Arguments:
     *     dataIndices -- a vector of (zero-base) indices of data to be plotted
     *     xVarIndex -- a (zero-base) index of x variable to be plotted
     *     yVarIndices -- a vector of (zero-base) indices of y variables to be plotted
     *     newPlotYesNo -- 1: data is plotted in a new figure,
     *                     0: plot is added in the current figure if there is
     *                        an open figure, otherwise a new figure is created
     * Output Arguments:
     *     polylineHandles ... handles of the plot polylines unglued under the axes
     *
     * Index arguments here are zero-base indices
     ****************************************************************/
     
    global PLT
    
    Data = PLT_Class_Data();
    
    // Convert zero-base indices to one-base indices
    dataIndices = dataIndices + 1;
    if ~isempty(xVarIndex) then
        xVarIndex = xVarIndex + 1;
    end
    yVarIndices = yVarIndices + 1;
    
    // Polyline property table
    PolylinePropTable = [..
        // {line_style[1-8], foreground[2-33], mark_style[0-9], mark_foreground[2-33]}
        1  2  0  2; // : 1
        2 13  0 13; // : 2
        3  5  0  5; // : 3
        4  9  0  9; // : 4
        5  6  0  6; // : 5
        6 16  0 16; // : 6
        7 20  0 20; // : 7
        8 23  0 23; // : 8
        1  3  0  3; // : 9
        2  4  0  4; // : 10
        3 19  0 19; // : 11
        4 28  0 28; // : 12
        5 32  0 32; // : 13
        6 18  0 18; // : 14
        7 25  0 25; // : 15
        8  7  0  7; // : 16
    ];
    
    max_id_prop = size(PolylinePropTable, 'r');
    
    // Axes information
    prevLegend = []; // existing legend text
    polylineIDs = []; // existing polyline IDs
    if newPlotYesNo ~= 0 then // new
        figureHandle = scf();
        clf(figureHandle);
        axesHandle = gca();
    else                    // add
        figureHandle = gcf();
        axesHandle = gca();
        for i = 1:length(axesHandle.children)
            if axesHandle.children(i).type == "Legend" then
                prevLegend = axesHandle.children(i).text;
            elseif axesHandle.children(i).type == "Polyline" then
                // polyline ID is stored in user_data
                polylineIDs($+1) = axesHandle.children(i).user_data;
            else
                // do nothing
            end
        end
    end
    
    figureHandle.figure_name = "Figure No.%d";
//    figureHandle.event_handler = "PLT_CB_handleFigureEvent";
//    figureHandle.event_handler_enable = "on";
    figureHandle.immediate_drawing = "off";
    //figureHandle.anti_aliasing = "16x"; // anti-aliasing activation will have no effect on most plateforms. (known incompatibilities)
    figureHandle.anti_aliasing = "off";
    axesHandle.tight_limits = "on";
    
    newLegend = []; // legend text to be added
    for j = 1:length(dataIndices)
        data = PLT.dataList.data(dataIndices(j));
        // Add plot
        polylineHandles = Data.plot(data, xVarIndex, yVarIndices, axesHandle);
        // Set properties
        for i = length(polylineHandles):-1:1
            // find an unused polyline ID -> id_polyline
            for k = 1:length(polylineIDs)+1
                if and(k ~= polylineIDs)
                    id_polyline = k;
                    break;
                end
            end
            // Set properties from the property table
            id_prop = modulo(id_polyline-1, max_id_prop) + 1;
            line_style = PolylinePropTable(id_prop, 1);
            foreground = PolylinePropTable(id_prop, 2);
            mark_style = PolylinePropTable(id_prop, 3);
            mark_foreground = PolylinePropTable(id_prop, 4);
            set(polylineHandles(i),..
                'line_style', line_style,..
                'foreground', foreground,..
                'mark_style', mark_style,..
                'mark_foreground', mark_foreground,..
                'mark_mode', 'off',..
                'user_data', id_polyline.. // store polyline ID in user_data
            );
            // Add polyline ID to the existing polyline IDs list
            polylineIDs($+1) = id_polyline;
        end
        // Construct legend text to be added
        for i = 1:length(yVarIndices)
            if isempty(xVarIndex) then
                legendstr(i) = data.name + ", y:" + data.varNames(yVarIndices(i));
            else
                legendstr(i) = data.name + ", x:" + data.varNames(xVarIndex) + ", y:" + data.varNames(yVarIndices(i));
            end
        end
        newLegend = [newLegend; legendstr];
    end
    
    // Update legend
    legendHandle = legend([prevLegend; newLegend], 'in_upper_right');
    
    // Store data bounds for later use
    axesHandle.user_data = axesHandle.data_bounds;
    
endfunction
