# This file is part of QuickPlot

#***********************************************************************
# Create Variable Dialog Box Module
#
#     Construct a dialog box for creating a variable
#     
# Public Procedures:
#     dbox_CreateVar -- create a dialog box for creating a variable
#
# Namespace:
#     ns_CreateVariable
#
# Private Procedures:
#     cb_btn_Apply -- callback to the "Apply" button
#     cb_btn_OK -- callback to the "OK" button
#     cb_btn_Cancel -- callback to the "Cancel" button
#     parse_expression -- parse expression
#***********************************************************************

#***********************************************************************
# dbox_CreateVar -- create a dialog box for creating a variable
#
# Parameter
#     reset -- whether to reset last input values
#***********************************************************************
proc dbox_CreateVar {{reset true}} {

	global DataNames; # list of all data names
	global SelectedDataIndices; # list of indices of currently selected data
	
	# Check if data exist
	if {[llength $DataNames] < 1} {
		tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
			-message "There is no data."
		return
	}
	
	# Reset some of the namespace variables
	if {$reset} {
		if {$SelectedDataIndices eq ""} {
			set ::ns_CreateVariable::dataName [lindex $DataNames 0]
		} else {
			set ::ns_CreateVariable::dataName [lindex $DataNames [lindex $SelectedDataIndices 0]]
		}
		set ::ns_CreateVariable::varName ""
		set ::ns_CreateVariable::expression ""
	}

	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
	# Create Variable Dialog Box
	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
	
	# Create a new top-level window for the dialog box
	toplevel .w_PLT_CreateVariable
	wm title .w_PLT_CreateVariable "Create Variable"
	wm iconbitmap .w_PLT_CreateVariable $::ICON_FILE
	bind .w_PLT_CreateVariable <Configure> {
		set ::ns_CreateVariable::geometry [winfo geometry .w_PLT_CreateVariable]
	}
	if {!$reset} {
		wm geometry .w_PLT_CreateVariable $::ns_CreateVariable::geometry
	}
	
	# In the dialog box, create "Setting" and "Buttons" frames
	set ::ns_CreateVariable::frameSetting [set frameSetting [frame .w_PLT_CreateVariable.f_Setting]]
	set ::ns_CreateVariable::frameButtons [set frameButtons [frame .w_PLT_CreateVariable.f_Buttons]]
	grid $frameSetting -padx 10 -pady 10 -sticky ew
	grid $frameButtons -padx 10 -pady 10 -sticky ew
	grid columnconfigure .w_PLT_CreateVariable 0 -weight 1

	# In Setting frame:
	label $frameSetting.lab_DataName -text "Data" -anchor w
	ttk::combobox $frameSetting.cmb_DataName -textvariable ::ns_CreateVariable::dataName -values $DataNames -state readonly
	label $frameSetting.lab_VarName -text "Variable name" -anchor w
	entry $frameSetting.ent_VarName -textvariable ::ns_CreateVariable::varName
	label $frameSetting.lab_Expression -text "Expression" -anchor w
	entry $frameSetting.ent_Expression -textvariable ::ns_CreateVariable::expression
	grid $frameSetting.lab_DataName $frameSetting.cmb_DataName -padx 4 -pady 2 -sticky ew
	grid $frameSetting.lab_VarName $frameSetting.ent_VarName -padx 4 -pady 2 -sticky ew
	grid $frameSetting.lab_Expression $frameSetting.ent_Expression -padx 4 -pady 2 -sticky ew
	grid columnconfigure $frameSetting 1 -weight 1

	# In Buttons frame:
	button $frameButtons.btn_Apply -text Apply -width 15 -command ::ns_CreateVariable::cb_btn_Apply
	button $frameButtons.btn_OK -text OK -width 15 -command ::ns_CreateVariable::cb_btn_OK
	button $frameButtons.btn_Cancel -text Cancel -width 15 -command ::ns_CreateVariable::cb_btn_Cancel
	pack $frameButtons.btn_Cancel $frameButtons.btn_Apply $frameButtons.btn_OK  -padx 10 -side right
	
	# Pop-up menu
	set pmenuExp [menu .w_PLT.popupMenuExp -tearoff false]
	$pmenuExp add command -label "Insert varval()" -command {
		dbox_InsertVarval $::ns_CreateVariable::frameSetting.ent_Expression $::ns_CreateVariable::dataName
	}
	bind $frameSetting.ent_Expression <ButtonPress-3> "tk_popup $pmenuExp %X %Y"
	
	# Set grab to this window
	grab set .w_PLT_CreateVariable
}

#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
# Create Variable Namespace
#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
namespace eval ns_CreateVariable {

	# Namespace variables
	variable frameSetting
	variable frameButtons
	variable dataName
	variable varName
	variable expression
	variable geometry
	
	#***********************************************************************
	# cb_btn_Apply -- callback to the "Apply" button
	#***********************************************************************
	proc cb_btn_Apply {} {
		global VarNamesList
		
		variable frameSetting
		variable varName
		variable expression
		
		if {$varName == ""} {
			tk_messageBox -parent .w_PLT_CreateVariable -type ok -icon error -title "Error!"\
				-message "\"Variable Name\" cannot be an empty string"
			error {Failed to create a variable}
		}
		
		if {$expression == ""} {
			tk_messageBox -parent .w_PLT_CreateVariable -type ok -icon error -title "Error!"\
				-message "\"Expression\" cannot be an empty string"
			error {Failed to create a variable}
		}
		
		set dataIndex [$frameSetting.cmb_DataName current]
		
		if {[lsearch -exact [lindex $VarNamesList $dataIndex] $varName] != -1} {
			tk_messageBox -parent .w_PLT_CreateVariable -type ok -icon error -title "Error!"\
				-message "Variable \"$varName\" already exists. Please choose a different variable name."
			error {Failed to create a variable}
		}
				
		# Parse expression
		set expression_parsed [::ns_CreateVariable::parse_expression $expression]
		if {$expression_parsed == ""} {
			error {Failed to create a variable}
		}
		
		# Create variable in Scilab.
		ScilabEval "PLT_Macro_createVar($dataIndex, '$varName', '$expression_parsed');" "seq"
	}
	
	#***********************************************************************
	# cb_btn_OK -- callback to the "OK" button
	#***********************************************************************
	proc cb_btn_OK {} {
		::ns_CreateVariable::cb_btn_Apply
		destroy .w_PLT_CreateVariable
		destroy .w_PLT.popupMenuExp
	}
	
	#***********************************************************************
	# cb_btn_Cancel -- callback to the "Cancel" button
	#***********************************************************************
	proc cb_btn_Cancel {} {
		destroy .w_PLT_CreateVariable
		destroy .w_PLT.popupMenuExp
	}
	
	#***********************************************************************
	# parse_expression -- parse expression
	#***********************************************************************
	proc parse_expression {expression} {
		set expression_parsed $expression
		
		# Double quatation characters
		regsub -all {'} $expression_parsed {''} expression_parsed
		regsub -all {"} $expression_parsed {""} expression_parsed
		
		return $expression_parsed
	}
}