# This file is part of QuickPlot

#***********************************************************************
# Delete Variable Dialog Box Module
#
#     Construct a dialog box for deleting variables
#
# Public Procedures:
#     dbox_DeleteVar -- create a dialog box for deleting variables
#
# Namespace:
#     ns_DeleteVariable
#
# Private Procedures:
#     update_var_listbox -- refresh the variable listbox according to the selected data
#     cb_btn_Delete -- callback to the "Delete" button
#     cb_btn_Close -- callback to the "Close" button
#***********************************************************************

#***********************************************************************
#  dbox_DeleteVar -- create a dialog box for deleting variables
#***********************************************************************
proc dbox_DeleteVar {} {

	global DataNames; # list of all data names
	global VarNamesList; # list of lists of variable names
	global SelectedDataIndices; # list of indices of currently selected data
	
	# Check if data exist
	if {[llength $DataNames] < 1} {
		tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
			-message "There is no data."
		return
	}

	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
	# Delete Variable Dialog Box
	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

	# Create a new top-level window for the dialog box
	toplevel .w_PLT_DeleteVariable
	wm title .w_PLT_DeleteVariable "Delete Variable"
	wm iconbitmap .w_PLT_DeleteVariable $::ICON_FILE

	# In the dialog box, create "Setting" and "Buttons" frames
	set ::ns_DeleteVariable::frameSetting [set frameSetting [frame .w_PLT_DeleteVariable.f_Setting]]
	set ::ns_DeleteVariable::frameButtons [set frameButtons [frame .w_PLT_DeleteVariable.f_Buttons]]
	grid $frameSetting -padx 10 -pady 10 -sticky news
	grid $frameButtons -padx 10 -pady 10 -sticky news
	grid columnconfigure .w_PLT_DeleteVariable 0 -weight 1
	grid rowconfigure .w_PLT_DeleteVariable 0 -weight 1

	# In Setting frame:
	label $frameSetting.lab_DataName -text "Data" -anchor w
	ttk::combobox $frameSetting.cmb_DataName -values $DataNames -state readonly
	bind $frameSetting.cmb_DataName <<ComboboxSelected>> ::ns_DeleteVariable::update_var_listbox
	if {$SelectedDataIndices eq ""} {
		$frameSetting.cmb_DataName current 0
	} else {
		$frameSetting.cmb_DataName current [lindex $SelectedDataIndices 0]
	}
	label $frameSetting.lab_Instruction -text "Select variables to delete" -anchor w
	listbox $frameSetting.lsb_Var -height 7\
		-xscrollcommand {$frameSetting.scrl_Var_h set}\
		-yscrollcommand {$frameSetting.scrl_Var_v set}\
		-selectmode extended -listvariable ::ns_DeleteVariable::varNames -activestyle none
	scrollbar $frameSetting.scrl_Var_h -orient horizontal\
		-command "$frameSetting.lsb_Var xview"
	scrollbar $frameSetting.scrl_Var_v -orient vertical\
		-command "$frameSetting.lsb_Var yview"
	grid $frameSetting.lab_DataName $frameSetting.cmb_DataName -padx 4 -pady 2 -sticky ew
	grid $frameSetting.lab_Instruction - -sticky w
	grid $frameSetting.lsb_Var - $frameSetting.scrl_Var_v -sticky news
	grid $frameSetting.scrl_Var_h - -sticky news
	grid columnconfigure $frameSetting 0 -weight 1
	grid rowconfigure $frameSetting 2 -weight 1
	$frameSetting.lsb_Var configure\
		-xscrollcommand [list sbset $frameSetting.scrl_Var_h]\
		-yscrollcommand [list sbset $frameSetting.scrl_Var_v]
	::ns_DeleteVariable::update_var_listbox
	
	# In Buttons frame:
	button $frameButtons.btn_Delete -text Delete -width 15 -command ::ns_DeleteVariable::cb_btn_Delete
	button $frameButtons.btn_Close -text Close -width 15 -command ::ns_DeleteVariable::cb_btn_Close
	pack $frameButtons.btn_Close $frameButtons.btn_Delete  -padx 10 -side right
	
	# Set grab to this window
	grab set .w_PLT_DeleteVariable
}

#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
# Delete Data Namespace
#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
namespace eval ns_DeleteVariable {
	
	# Namespace variables
	variable frameSetting
	variable frameButtons
	variable varNames
	
	#***********************************************************************
	# update_var_listbox -- refresh the variable listbox according to the selected data
	#***********************************************************************
	proc update_var_listbox {} {
		global VarNamesList; # list of lists of variable names
		variable frameSetting
		variable varNames
		
		set dataIndex [$frameSetting.cmb_DataName current]
		set varNames [lindex $VarNamesList $dataIndex]
	}
	
	#***********************************************************************
	#  cb_btn_Delete -- callback to the "Delete" button
	#***********************************************************************
	proc cb_btn_Delete {} {
		global VarNamesList; # list of lists of variable names
		
		variable frameSetting
		variable varNames
		
		set dataIndex [$frameSetting.cmb_DataName current]
		set varIndices [$frameSetting.lsb_Var curselection]
		
		if {$varIndices eq ""} {
			tk_messageBox -parent .w_PLT_DeleteVariable -type ok -icon error -title "Error!"\
				-message "No variable is selected."
			return
		}
		
		for {set i 0} {$i < [llength $varIndices]} {incr i} {
			set varIndex [lindex $varIndices end-$i]
			ScilabEval "PLT_Macro_deleteVar($dataIndex, $varIndex);" "seq"
			set varNames [lreplace $varNames $varIndex $varIndex]
		}
		lset VarNamesList $dataIndex $varNames
		cb_filter_xvar
		cb_filter_yvar
		cb_ckb_Browse
	}
	
	#***********************************************************************
	#  cb_btn_Close -- callback to the "Close" button
	#***********************************************************************
	proc cb_btn_Close {} {
		destroy .w_PLT_DeleteVariable
	}
}