# This file is part of QuickPlot

#***********************************************************************
# Edit Preset Filters Dialog Box Module
#
#     Construct a dialog box for editing pre-set filter patterns
#
# Public Procedures:
#     dbox_EditPresetFilters -- create a dialog box for editing pre-set filter patterns
#
# Namespace:
#     ns_EditPresetFilters
#
# Private Procedures:
#     update_FilterPatterns -- refresh the filter patterns in the text widget
#     cb_btn_OK -- callback to the "OK" button
#     cb_btn_Cancel -- callback to the "Cancel" button
#***********************************************************************

#***********************************************************************
# dbox_EditPresetFilters -- create a dialog box for editing pre-set filter patterns
#***********************************************************************
proc dbox_EditPresetFilters {} {
	
	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
	# Pre-set Filter Dialog Box
	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

	# Create a new top-level window for the dialog box
	toplevel .w_PLT_PresetFilter
	wm title .w_PLT_PresetFilter "Edit Preset Filters"
	wm iconbitmap .w_PLT_PresetFilter $::ICON_FILE

	# In the dialog box, create "DataFilter", "VarFilter" and "Buttons" frames
	set ::ns_EditPresetFilters::frameDataFilter [set frameDataFilter [frame .w_PLT_PresetFilter.f_DataFilter]]
	set ::ns_EditPresetFilters::frameVarFilter [set frameVarFilter [frame .w_PLT_PresetFilter.f_VarFilter]]
	set ::ns_EditPresetFilters::frameButtons [set frameButtons [frame .w_PLT_PresetFilter.f_Buttons]]
	grid $frameDataFilter -padx 10 -pady 10 -sticky news
	grid $frameVarFilter -padx 10 -pady 10 -sticky news
	grid $frameButtons -padx 10 -pady 10 -sticky news
	grid columnconfigure .w_PLT_PresetFilter 0 -weight 1
	grid rowconfigure .w_PLT_PresetFilter {0 1} -weight 1

	# In DataFilter frame:
	label $frameDataFilter.lab_FilterPatterns -text "Data filter" -anchor w
	text $frameDataFilter.txt_FilterPatterns -wrap none -relief sunken -height 7 -width 30\
		-xscrollcommand {$frameDataFilter.scrl_FilterPatterns_h set}\
		-yscrollcommand {$frameDataFilter.scrl_FilterPatterns_v set}
	scrollbar $frameDataFilter.scrl_FilterPatterns_h -orient horizontal\
		-command "$frameDataFilter.txt_FilterPatterns xview"
	scrollbar $frameDataFilter.scrl_FilterPatterns_v -orient vertical\
		-command "$frameDataFilter.txt_FilterPatterns yview"
	grid $frameDataFilter.lab_FilterPatterns -sticky w
	grid $frameDataFilter.txt_FilterPatterns $frameDataFilter.scrl_FilterPatterns_v -sticky news
	grid $frameDataFilter.scrl_FilterPatterns_h -sticky news
	grid columnconfigure $frameDataFilter 0 -weight 1
	grid rowconfigure $frameDataFilter 1 -weight 1
	$frameDataFilter.txt_FilterPatterns configure\
		-xscrollcommand [list sbset $frameDataFilter.scrl_FilterPatterns_h]\
		-yscrollcommand [list sbset $frameDataFilter.scrl_FilterPatterns_v]
	
	# In VarFilter frame:
	label $frameVarFilter.lab_FilterPatterns -text "Variable filter" -anchor w
	text $frameVarFilter.txt_FilterPatterns -wrap none -relief sunken -height 7 -width 30\
		-xscrollcommand {$frameVarFilter.scrl_FilterPatterns_h set}\
		-yscrollcommand {$frameVarFilter.scrl_FilterPatterns_v set}
	scrollbar $frameVarFilter.scrl_FilterPatterns_h -orient horizontal\
		-command "$frameVarFilter.txt_FilterPatterns xview"
	scrollbar $frameVarFilter.scrl_FilterPatterns_v -orient vertical\
		-command "$frameVarFilter.txt_FilterPatterns yview"
	grid $frameVarFilter.lab_FilterPatterns -sticky w
	grid $frameVarFilter.txt_FilterPatterns $frameVarFilter.scrl_FilterPatterns_v -sticky news
	grid $frameVarFilter.scrl_FilterPatterns_h -sticky news
	grid columnconfigure $frameVarFilter 0 -weight 1
	grid rowconfigure $frameVarFilter 1 -weight 1
	$frameVarFilter.txt_FilterPatterns configure\
		-xscrollcommand [list sbset $frameVarFilter.scrl_FilterPatterns_h]\
		-yscrollcommand [list sbset $frameVarFilter.scrl_FilterPatterns_v]
		
	# Refresh the filter patterns:
	::ns_EditPresetFilters::update_FilterPatterns

	# In Buttons frame:
	button $frameButtons.btn_OK -text OK -width 15 -command ::ns_EditPresetFilters::cb_btn_OK
	button $frameButtons.btn_Cancel -text Cancel -width 15 -command ::ns_EditPresetFilters::cb_btn_Cancel
	pack $frameButtons.btn_Cancel $frameButtons.btn_OK  -padx 10 -side right

	# Set grab to this window
	grab set .w_PLT_PresetFilter
}

#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
# Pre-set Filter Namespace
#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
namespace eval ns_EditPresetFilters {

	# Namespace variables
	variable frameDataFilter
	variable frameVarFilter
	variable frameButtons
	
	#***********************************************************************
	# update_FilterPatterns -- refresh the filter patterns in the text widget
	#***********************************************************************
	proc update_FilterPatterns {} {
		global PresetDataFilterPatterns; # preset data filter patterns
		global PresetVarFilterPatterns; # preset variable filter patterns 
		
		variable frameDataFilter
		variable frameVarFilter
		
		$frameDataFilter.txt_FilterPatterns delete 0.0 end
		foreach pattern $PresetDataFilterPatterns {
			$frameDataFilter.txt_FilterPatterns insert end $pattern
			$frameDataFilter.txt_FilterPatterns insert end "\n"
		}
		
		$frameVarFilter.txt_FilterPatterns delete 0.0 end
		foreach pattern $PresetVarFilterPatterns {
			$frameVarFilter.txt_FilterPatterns insert end $pattern
			$frameVarFilter.txt_FilterPatterns insert end "\n"
		}
	}
	
	#***********************************************************************
	# cb_btn_OK -- callback to the "OK" button
	#***********************************************************************
	proc cb_btn_OK {} {
		global PresetDataFilterPatterns; # preset data filter patterns
		global PresetVarFilterPatterns; # preset variable filter patterns
		
		variable frameDataFilter
		variable frameVarFilter
		
		set PresetDataFilterPatterns {}
		set line [$frameDataFilter.txt_FilterPatterns get 1.0 1.end]
		for {set i 2} {[string length $line] > 0 } {incr i} {
			lappend PresetDataFilterPatterns $line
			set line [$frameDataFilter.txt_FilterPatterns get $i.0 $i.end]
		}
		$::frameData.cmb_DataFilter configure -values $PresetDataFilterPatterns
		
		set PresetVarFilterPatterns {}
		set line [$frameVarFilter.txt_FilterPatterns get 1.0 1.end]
		for {set i 2} {[string length $line] > 0 } {incr i} {
			lappend PresetVarFilterPatterns $line
			set line [$frameVarFilter.txt_FilterPatterns get $i.0 $i.end]
		}
		$::frameXVar.cmb_XVarFilter configure -values $PresetVarFilterPatterns
		$::frameYVar.cmb_YVarFilter configure -values $PresetVarFilterPatterns
		
		destroy .w_PLT_PresetFilter
	}
	
	#***********************************************************************
	# cb_btn_Cancel -- callback to the "Cancel" button
	#***********************************************************************
	proc cb_btn_Cancel {} {
		destroy .w_PLT_PresetFilter
	}
}