# This file is part of QuickPlot

#***********************************************************************
# Edit Variable Names Dialog Box Module
#
#     Construct a dialog box for editing variable names
#
# Public Procedures:
#     dbox_EditVarNames -- create a dialog box for editing variable names
#
# Namespace:
#     ns_EditVarNames
#
# Private Procedures:
#     update_varnames_editor -- refresh variable names in editor text widget according to the selected data
#     cb_btn_OK -- callback to the "OK" button
#     cb_btn_Cancel -- callback to the "Cancel" button
#***********************************************************************

#***********************************************************************
# dbox_EditVarNames -- create a dialog box for editing variable names
#     reset -- whether to reset last input values
#***********************************************************************
proc dbox_EditVarNames {{reset true}} {

	global DataNames; # list of all data names
	global VarNamesList; # list of lists of variable names
	global SelectedDataIndices; # list of indices of currently selected data
	
	# Check if data exist
	if {[llength $DataNames] < 1} {
		tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
			-message "There is no data."
		return
	}

	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
	# Edit Variable Names Dialog Box
	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

	# Create a new top-level window for the dialog box
	toplevel .w_PLT_EditVarNames
	wm title .w_PLT_EditVarNames "Edit Variable Names"
	wm iconbitmap .w_PLT_EditVarNames $::ICON_FILE

	# In the dialog box, create "Setting" and "Buttons" frames
	set ::ns_EditVarNames::frameSetting [set frameSetting [frame .w_PLT_EditVarNames.f_Setting]]
	set ::ns_EditVarNames::frameButtons [set frameButtons [frame .w_PLT_EditVarNames.f_Buttons]]
	grid $frameSetting -padx 10 -pady 10 -sticky news
	grid $frameButtons -padx 10 -pady 10 -sticky news
	grid columnconfigure .w_PLT_EditVarNames 0 -weight 1
	grid rowconfigure .w_PLT_EditVarNames 0 -weight 1

	# In Setting frame:
	label $frameSetting.lab_DataName -text "Data" -anchor w
	ttk::combobox $frameSetting.cmb_DataName -values $DataNames -state readonly
	bind $frameSetting.cmb_DataName <<ComboboxSelected>> ::ns_EditVarNames::update_varnames_editor
	if {$SelectedDataIndices eq ""} {
		$frameSetting.cmb_DataName current 0
	} else {
		$frameSetting.cmb_DataName current [lindex $SelectedDataIndices 0]
	}
	label $frameSetting.lab_VarNames -text "Variable names" -anchor nw
	frame $frameSetting.f_VarNameEditor; # VarName-Editor frame
	grid $frameSetting.lab_DataName $frameSetting.cmb_DataName -padx 4 -pady 2 -sticky ew
	grid $frameSetting.lab_VarNames $frameSetting.f_VarNameEditor -padx 4 -pady 2 -sticky news
	grid columnconfigure $frameSetting 1 -weight 1
	grid rowconfigure $frameSetting 1 -weight 1

	# In VarName-Editor frame
	text $frameSetting.f_VarNameEditor.txt_VarNameEditor -wrap none -relief sunken -height 7 -width 30\
		-xscrollcommand {$frameSetting.f_VarNameEditor.scrl_VarNameEditor_h set}\
		-yscrollcommand {$frameSetting.f_VarNameEditor.scrl_VarNameEditor_v set}
	scrollbar $frameSetting.f_VarNameEditor.scrl_VarNameEditor_h -orient horizontal\
		-command "$frameSetting.f_VarNameEditor.txt_VarNameEditor xview"
	scrollbar $frameSetting.f_VarNameEditor.scrl_VarNameEditor_v -orient vertical\
		-command "$frameSetting.f_VarNameEditor.txt_VarNameEditor yview"
	grid $frameSetting.f_VarNameEditor.txt_VarNameEditor $frameSetting.f_VarNameEditor.scrl_VarNameEditor_v -sticky news
	grid $frameSetting.f_VarNameEditor.scrl_VarNameEditor_h -sticky news
	grid columnconfigure $frameSetting.f_VarNameEditor 0 -weight 1
	grid rowconfigure $frameSetting.f_VarNameEditor 0 -weight 1
	$frameSetting.f_VarNameEditor.txt_VarNameEditor configure\
		-xscrollcommand [list sbset $frameSetting.f_VarNameEditor.scrl_VarNameEditor_h]\
		-yscrollcommand [list sbset $frameSetting.f_VarNameEditor.scrl_VarNameEditor_v]
	::ns_EditVarNames::update_varnames_editor

	# In Buttons frame:
	button $frameButtons.btn_OK -text OK -width 15 -command ::ns_EditVarNames::cb_btn_OK
	button $frameButtons.btn_Cancel -text Cancel -width 15 -command ::ns_EditVarNames::cb_btn_Cancel
	pack $frameButtons.btn_Cancel $frameButtons.btn_OK  -padx 10 -side right

	# Set grab to this window
	grab set .w_PLT_EditVarNames
}

#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
# Edit Variable Names Namespace
#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
namespace eval ns_EditVarNames {

	# Namespace variables
	variable frameSetting
	variable frameButtons

	#***********************************************************************
	# update_varnames_editor -- refresh variable names in editor text widget according to the selected data
	#***********************************************************************
	proc update_varnames_editor {} {
	
		global VarNamesList; # list of lists of variable names
		variable frameSetting
		
		set dataIndex [$frameSetting.cmb_DataName current]
		set varNames [lindex $VarNamesList $dataIndex]
		$frameSetting.f_VarNameEditor.txt_VarNameEditor delete 0.0 end
		foreach varName $varNames {
			$frameSetting.f_VarNameEditor.txt_VarNameEditor insert end $varName
			$frameSetting.f_VarNameEditor.txt_VarNameEditor insert end "\n"
		}
	}
	
	#***********************************************************************
	# cb_btn_OK -- callback to the "OK" button
	#***********************************************************************
	proc cb_btn_OK {} {
	
		global VarNamesList; # list of lists of variable names
	
		variable frameSetting
		
		set dataIndex [$frameSetting.cmb_DataName current]
		set nCol [llength [lindex $VarNamesList $dataIndex]]
		set varNames {}
		set row 1
		
		while {1} {
			if {$row <= $nCol} {
				set varName [string trim [$frameSetting.f_VarNameEditor.txt_VarNameEditor get $row.0 $row.end]]
				if {[string length $varName] == 0} {
					tk_messageBox -parent .w_PLT_EditVarNames -type ok -icon error -title "Error!"\
						-message "Variable name cannot be an empty string at row $row."
					error {Failed to edit variable names}
				}
				if {[lsearch -exact $varNames $varName] != -1} {
					tk_messageBox -parent .w_PLT_EditVarNames -type ok -icon error -title "Error!"\
						-message "There is a duplicated variable name \"$varName\" at row $row. Please choose a unique name for each variable."
					error {Failed to edit variable names}
				}
				lappend varNames $varName
			} else {
				set residue [$frameSetting.f_VarNameEditor.txt_VarNameEditor get $row.0 end]
				regsub -all {\s} $residue {} residue
				if {[string length $residue] > 0} {
					tk_messageBox -parent .w_PLT_EditVarNames -type ok -icon error -title "Error!"\
						-message "The number of variable names must match the number of columns of data ($nCol)."
					error {Failed to edit variable names}
				}
				break
			}
			incr row
		}
		
		# Set new variable names in Scilab.
		ScilabEval "PLT_Macro_setVarNames($dataIndex, [list2strvec $varNames]);" "seq"
		
		destroy .w_PLT_EditVarNames
	}
	
	#***********************************************************************
	# cb_btn_Cancel -- callback to the "Cancel" button
	#***********************************************************************
	proc cb_btn_Cancel {} {
		destroy .w_PLT_EditVarNames
	}
}