# This file is part of QuickPlot

#***********************************************************************
# Import Data Dialog Box Module
#
#     Construct a dialog box for importing data from file
#
# Public Procedures:
#     dbox_ImportData -- create a dialog box for importing data from a file
#
# Namespace:
#     ns_ImportData
#
# Private Procedures:
#     preview_load -- read data from file for preview
#     preview_show -- print the preview data in the preview text widget
#     preview_varNameRow -- highlight the variable name row in the preview
#     preview_skipRows -- draw a border line after the skipped rows in the preview
#     cb_btn_getOpen_ImportFile -- callback to the "open file" button
#     cb_spn_SkipRows -- callback to the "skip rows" spin box
#     cb_cmb_Separator -- callback to the "separator" combo box
#     cb_ckb_VarNamesFromRow -- callback to the "variable names from row" check box
#     cb_spn_VarNameRow -- callback to the "variable name row" spin box
#     cb_btn_Import -- callback to the "import" button
#     cb_btn_Cancel -- callback to the "cancel" button
#***********************************************************************

#***********************************************************************
# dbox_ImportData -- create a dialog box for importing data from a file 
#***********************************************************************
proc dbox_ImportData {} {

	# Reset some of the namespace variables
	set ::ns_ImportData::importFileList ""; # paths of the files from which data is imported
	set ::ns_ImportData::previewData ""; # a list each of whose elements is a line of data from file

	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
	# Import Data Dialog Box
	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

	# Create a new top-level window for the dialog box
	toplevel .w_PLT_ImportData
	wm title .w_PLT_ImportData "Import Data from File"
	wm iconbitmap .w_PLT_ImportData $::ICON_FILE

	# In the dialog box, create "Setting", "Preview", and "Buttons" frames
	set ::ns_ImportData::frameSetting [set frameSetting [frame .w_PLT_ImportData.f_Setting]]
	set ::ns_ImportData::framePreview\
		[set framePreview [labelframe .w_PLT_ImportData.f_Preview -text "Preview (first file, first $::ns_ImportData::maxPreviewRows rows)"]]
	set ::ns_ImportData::frameButtons [set frameButtons [frame .w_PLT_ImportData.f_Buttons]]
	grid $frameSetting -padx 10 -pady 10 -sticky ew
	grid $framePreview -padx 10 -pady 10 -sticky news
	grid $frameButtons -padx 10 -pady 10 -sticky ew
	grid rowconfigure .w_PLT_ImportData 1 -weight 1
	grid columnconfigure .w_PLT_ImportData 0 -weight 1
	
	# In Setting frame:
	label $frameSetting.lab_ImportFile -text "File" -anchor w
	entry $frameSetting.ent_ImportFile -textvariable ::ns_ImportData::importFileList -state readonly
	button $frameSetting.btn_getOpen_ImportFile -text "Browse" -command ::ns_ImportData::cb_btn_getOpen_ImportFile
	label $frameSetting.lab_SkipRows -text "Skip rows" -anchor w
	spinbox $frameSetting.spn_SkipRows -from 0 -to 10000 -increment 1 -textvariable ::ns_ImportData::skipRows\
		-command ::ns_ImportData::cb_spn_SkipRows
	bind $frameSetting.spn_SkipRows <KeyRelease> ::ns_ImportData::cb_spn_SkipRows
	label $frameSetting.lab_Separator -text "Separator" -anchor w
	ttk::combobox $frameSetting.cmb_Separator -textvariable ::ns_ImportData::separatorSelect\
		-values $::SEPARATOR_LIST -state readonly; # see "sep_char" function for SEPARATOR_LIST
	bind $frameSetting.cmb_Separator <<ComboboxSelected>> ::ns_ImportData::cb_cmb_Separator
	bind $frameSetting.cmb_Separator <MouseWheel> break
	checkbutton $frameSetting.ckb_VarNamesFromRow -text "Import variable names from row"\
		-variable ::ns_ImportData::varNamesFromRowYesNo -anchor w -command ::ns_ImportData::cb_ckb_VarNamesFromRow
	label $frameSetting.lab_VarNameRow -text "Row of variable names" -anchor w
	spinbox $frameSetting.spn_VarNameRow -from 1 -to 10000 -increment 1 -textvariable ::ns_ImportData::varNameRow\
		-command ::ns_ImportData::cb_spn_VarNameRow
	if {$::ns_ImportData::varNamesFromRowYesNo == 1} {
		$frameSetting.lab_VarNameRow configure -state normal
		$frameSetting.spn_VarNameRow configure -state normal
	} else {
		$frameSetting.lab_VarNameRow configure -state disabled
		$frameSetting.spn_VarNameRow configure -state disabled
	}
	bind $frameSetting.spn_VarNameRow <KeyRelease> ::ns_ImportData::cb_spn_VarNameRow
	grid $frameSetting.lab_ImportFile $frameSetting.ent_ImportFile - $frameSetting.btn_getOpen_ImportFile\
		-padx 4 -pady 2 -sticky ew
	grid $frameSetting.lab_SkipRows $frameSetting.spn_SkipRows -padx 4 -pady 2 -sticky w
	grid $frameSetting.lab_Separator $frameSetting.cmb_Separator -padx 4 -pady 2 -sticky w
	grid $frameSetting.ckb_VarNamesFromRow -columnspan 2 -sticky w
	grid $frameSetting.lab_VarNameRow $frameSetting.spn_VarNameRow -padx 4 -pady 2 -sticky w
	grid columnconfigure $frameSetting 2 -weight 1

	# In Preview frame:
	text $framePreview.txt_Preview -wrap none -relief sunken\
		-xscrollcommand {$framePreview.scrl_Preview_h set}\
		-yscrollcommand {$framePreview.scrl_Preview_v set}\
		-bg {light gray} -state disabled -exportselection no
	bind $framePreview.txt_Preview <ButtonPress> break
	scrollbar $framePreview.scrl_Preview_h -orient horizontal -command "$framePreview.txt_Preview xview"
	scrollbar $framePreview.scrl_Preview_v -orient vertical -command "$framePreview.txt_Preview yview"
	grid $framePreview.txt_Preview $framePreview.scrl_Preview_v -sticky news
	grid $framePreview.scrl_Preview_h -sticky news
	grid rowconfigure $framePreview 0 -weight 1
	grid columnconfigure $framePreview 0 -weight 1
	$framePreview.txt_Preview configure\
		-xscrollcommand [list sbset $framePreview.scrl_Preview_h]\
		-yscrollcommand [list sbset $framePreview.scrl_Preview_v]
	
	# In Buttons frame:
	button $frameButtons.btn_Import -text Import -width 15 -command ::ns_ImportData::cb_btn_Import
	button $frameButtons.btn_Cancel -text Cancel -width 15 -command ::ns_ImportData::cb_btn_Cancel
	pack $frameButtons.btn_Cancel $frameButtons.btn_Import -padx 10 -side right
	
	# Set grab to this window
	grab set .w_PLT_ImportData
}

#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
# Import Data Namespace
#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
namespace eval ns_ImportData {
	
	# Namespace variables
	variable frameSetting
	variable framePreview
	variable frameButtons
	variable importFileList ""; # paths of the files from which data is imported
	variable skipRows 0; # number of rows to be skipped
	variable separatorSelect "space/tab"; # separator choice from SEPARATOR_LIST
	variable varNamesFromRowYesNo 0; # flag: whether to import variable names from the same file or not
	variable varNameRow 1; # row number where variable names reside
	variable maxPreviewRows 50; # maximum number of rows displayed in the preview region
	variable textPreviewColumn; # array of text widget handles for preview
	variable previewFontFamily "courier"; # font family for preview
	variable previewFontSize 10; # font size for preview
	variable previewData ""; # a list each of whose elements is a line of data from file
	variable nColPreview 0; # the number of columns in the preview text widget including the line number column

	#***********************************************************************
	# preview_load -- read data from file for preview
	#
	# Parameter:
	#     filename -- path of the file from which data is imported
	#***********************************************************************
	proc preview_load {filename} {
		variable maxPreviewRows; # maximum number of rows displayed in the preview region
		variable previewData; # a list each of whose elements is a line of data from file
		
		set previewData ""
		set fID [open $filename]
		set rowNum 1
		while {[gets $fID line] >= 0 && $rowNum <= $maxPreviewRows} {
			lappend previewData $line
			incr rowNum
		}
		close $fID
	}
	
	#***********************************************************************
	# preview_show -- print the preview data in the preview text widget
	#***********************************************************************
	proc preview_show {} {
		variable framePreview
		variable separatorSelect; # separator choice from SEPARATOR_LIST
		variable textPreviewColumn; # array of text widget handles for preview
		variable previewFontFamily; # font family for preview
		variable previewFontSize; # font size for preview
		variable previewData; # a list each of whose elements is a line of data from file
		variable nColPreview; # the number of columns in the preview text widget including the line number column
		
		$framePreview.txt_Preview configure -state normal
		$framePreview.txt_Preview delete 1.0 end
		
		if {$previewData ne ""} {
		
			# Create Column#0 which is used to display row numbers
			set textPreviewColumn(0) [text $framePreview.txt_Preview.txt_Column0 -wrap none -relief flat\
				-font "$previewFontFamily $previewFontSize normal"]
			$framePreview.txt_Preview window create end -window $textPreviewColumn(0)
			$textPreviewColumn(0) configure -bg {gray} -fg {white} -height 1 -width 1
			bind $textPreviewColumn(0) <ButtonPress> break
			bind $textPreviewColumn(0) <MouseWheel> break
			$textPreviewColumn(0) tag configure "row-number" -justify right
			$textPreviewColumn(0) tag configure "skiprows-border" -foreground red

			set maxColNum 0
			set rowNum 1
			foreach line "$previewData" {
				switch -- $separatorSelect {
					"space/tab" {set elements [regexp -all -inline {\S+} $line]}
					"comma" {set elements [split $line {,}]}
				}
				# Trim each element
				set trimmedElements [list]
				foreach element $elements {
					lappend trimmedElements [string trim $element]
				}
				set elements $trimmedElements
				
				# Add Column#0 a row number
				$textPreviewColumn(0) configure -state normal
				$textPreviewColumn(0) configure -height [expr {$rowNum + 1}]; # +1 for the skiprows-border
				$textPreviewColumn(0) insert end "$rowNum\n" "row-number"
				$textPreviewColumn(0) configure -width [expr {max([string length $rowNum],[lindex [$textPreviewColumn(0) configure -width] 4])}]
				$textPreviewColumn(0) configure -state disabled

				# Column#j holds the j-th column of data
				set maxColNum [expr {max([llength $elements],$maxColNum)}]
				for {set j 1} {$j <= $maxColNum} {incr j} {
				
					# If Column#j does not exist, then create it
					if {[winfo exists $framePreview.txt_Preview.txt_Column$j] == 0} {
						set textPreviewColumn($j) [text $framePreview.txt_Preview.txt_Column$j -wrap none -relief solid\
							-font "$previewFontFamily $previewFontSize normal"]
						$framePreview.txt_Preview window create end -window $textPreviewColumn($j)
						$textPreviewColumn($j) configure -bg {light gray} -height 1 -width 1
						bind $textPreviewColumn($j) <ButtonPress> break
						bind $textPreviewColumn($j) <MouseWheel> break
						$textPreviewColumn($j) tag configure "skiprows-border" -foreground red
						$textPreviewColumn($j) tag configure "skipped" -foreground gray
						$textPreviewColumn($j) tag configure "highlight" -foreground blue -font "$previewFontFamily $previewFontSize {bold italic}"
						$textPreviewColumn($j) insert end "[string repeat "\n" [expr {$rowNum-1}]]"
					}
					
					# Add Column#j a data element
					$textPreviewColumn($j) configure -state normal
					$textPreviewColumn($j) configure -height [expr {$rowNum + 1}]; # +1 for the skiprows-border
					set elem_repTab [string map {\t <TAB>} [lindex $elements [expr {$j-1}]]]; # A tab character is replaced with "<TAB>" for convenience
					$textPreviewColumn($j) insert end "$elem_repTab\n"
					$textPreviewColumn($j) configure -width [expr {max([string length $elem_repTab],[lindex [$textPreviewColumn($j) configure -width] 4])}]
					$textPreviewColumn($j) configure -state disabled
				}
				incr rowNum
			}
			set nColPreview [expr {$maxColNum + 1}]
			::ns_ImportData::preview_skipRows
			::ns_ImportData::preview_varNameRow
		}
		$framePreview.txt_Preview configure -state disabled
	}
	
	#***********************************************************************
	# preview_varNameRow -- highlight the variable name row in the preview
	#***********************************************************************
	proc preview_varNameRow {} {
		variable skipRows; # number of rows to be skipped
		variable varNamesFromRowYesNo; # flag: whether to import variable names from the same file or not
		variable varNameRow; # row number where variable names reside
		variable textPreviewColumn; # array of text widget handles for preview
		variable nColPreview; # the number of columns in the preview text widget including the line number column
		
		# For each column remove old highlights and set new highlights at the variable name row
		for {set j 1} {$j < $nColPreview} {incr j} {
			$textPreviewColumn($j) configure -state normal
			$textPreviewColumn($j) tag remove "highlight" 1.0 end
			if {$varNamesFromRowYesNo == 1} {
				if {$varNameRow <= $skipRows} {
					$textPreviewColumn($j) tag add "highlight" $varNameRow.0 "$varNameRow.0 lineend"
				} else {
					$textPreviewColumn($j) tag add "highlight" "$varNameRow.0 + 1 line" "$varNameRow.0 + 1 line lineend"
				}
			}
			$textPreviewColumn($j) configure -state disabled
		}
	}
	
	#***********************************************************************
	# preview_skipRows -- draw a border line after the skipped rows in the preview
	#***********************************************************************
	proc preview_skipRows {} {
		variable skipRows; # number of rows to be skipped
		variable textPreviewColumn; # array of text widget handles for preview
		variable nColPreview; # the number of columns in the preview text widget including the line number column
		
		# For each column...
		for {set j 0} {$j < $nColPreview} {incr j} {
			$textPreviewColumn($j) configure -state normal
			
			# Remove old skiprows-border and set new skiprows-border
			if {[$textPreviewColumn($j) tag ranges "skiprows-border"] ne ""} {
				$textPreviewColumn($j) delete "skiprows-border.first" "skiprows-border.last"
			}
			set columnWidth [lindex [$textPreviewColumn($j) configure -width] 4]
			$textPreviewColumn($j) insert "[expr {$skipRows + 1}].0" "[string repeat {-} $columnWidth]\n" "skiprows-border"
			
			# Remove "skipped" tag from everywhere and set it to the new skipped region
			$textPreviewColumn($j) tag remove "skipped" 1.0 end
			if {$skipRows > 0} {
				$textPreviewColumn($j) tag add "skipped" 1.0 "$skipRows.0 lineend"
			}
			
			$textPreviewColumn($j) configure -state disabled
		}
	}
	
	#***********************************************************************
	# cb_btn_getOpen_ImportFile -- callback to the "open file" button
	#***********************************************************************
	proc cb_btn_getOpen_ImportFile {} {
		global CurrentDir; # current directory path
		variable importFileList; # paths of the files from which data is imported
		variable separatorSelect; # separator choice from SEPARATOR_LIST
		
		set typelist {
			{"All Files" {*}}
			{"Text Files" {".txt"}}
			{"CSV Files" {".csv"}}
		}
		set filePath [tk_getOpenFile -initialdir $CurrentDir -multiple true -parent .w_PLT_ImportData -filetypes $typelist]
		if {$filePath ne ""} {
			set importFileList $filePath
			# If all files have .csv extention, set separator to be comma.
			if {[llength [filter_list $importFileList {*.csv}]] == [llength $importFileList]} {
				set separatorSelect "comma"
			}
			::ns_ImportData::preview_load [lindex $importFileList 0]
			::ns_ImportData::preview_show
		}
	}
	
	#***********************************************************************
	# cb_spn_SkipRows -- callback to the "skip rows" spin box
	#***********************************************************************
	proc cb_spn_SkipRows {} {
		variable skipRows; # number of rows to be skipped
		
		if {[string is integer -strict $skipRows] == 0 || $skipRows < 0} {
			set skipRows 0
		}
		::ns_ImportData::preview_skipRows
	}
	
	#***********************************************************************
	# cb_cmb_Separator -- callback to the "separator" combo box
	#***********************************************************************
	proc cb_cmb_Separator {} {
		::ns_ImportData::preview_show
	}
	
	#***********************************************************************
	# cb_ckb_VarNamesFromRow -- callback to the "variable names from row" check box
	#***********************************************************************
	proc cb_ckb_VarNamesFromRow {} {
		variable frameSetting
		variable varNamesFromRowYesNo; # flag: whether to import variable names from the same file or not
		
		if {$varNamesFromRowYesNo == 1} {
			$frameSetting.lab_VarNameRow configure -state normal
			$frameSetting.spn_VarNameRow configure -state normal
		} else {
			$frameSetting.lab_VarNameRow configure -state disabled
			$frameSetting.spn_VarNameRow configure -state disabled
		}
		::ns_ImportData::preview_varNameRow
	}
	
	#***********************************************************************
	# cb_spn_VarNameRow -- callback to the "variable name row" spin box
	#***********************************************************************
	proc cb_spn_VarNameRow {} {
		variable varNameRow; # row number where variable names reside
		
		if {[string is integer -strict $varNameRow] == 0 || $varNameRow < 1} {
			set varNameRow 1
		}
		::ns_ImportData::preview_varNameRow
	}
	
	#***********************************************************************
	# cb_btn_Import -- callback to the "import" button
	#***********************************************************************
	proc cb_btn_Import {} {
		global DataNames; # list of all data names
		global VarNamesList; # list of lists of variable names
	
		variable importFileList; # paths of the files from which data is imported
		variable skipRows; # number of rows to be skipped
		variable separatorSelect; # separator choice from SEPARATOR_LIST
		variable varNamesFromRowYesNo; # flag: whether to import variable names from the same file or not
		variable varNameRow; # row number where variable names reside
		
		# Create a progression bar
		ScilabEval "wid = progressionbar('Reading data from file...');" "seq"
		ScilabEval "progressionbar(wid);" "seq"
		set close_progression_bar {ScilabEval "close(wid);" "seq"}; # eval this variable to close the progression bar
	
		# Error if no file is chosen
		if {$importFileList == ""} {
			eval $close_progression_bar
			tk_messageBox -parent .w_PLT_ImportData -type ok -icon error -title "Error!"\
				-message "Please choose a file."
			return
		}
			
		foreach importFile $importFileList {
		
			# Check input errors
			if {[file exists $importFile] != 1} {
				eval $close_progression_bar
				tk_messageBox -parent .w_PLT_ImportData -type ok -icon error -title "Error!"\
					-message "File \"$importFile\" doesn't exist or isn't accessible."
				return
			}
			if {[file size $importFile] == 0} {
				eval $close_progression_bar
				tk_messageBox -parent .w_PLT_ImportData -type ok -icon error -title "Error!"\
					-message "File \"$importFile\" is empty."
				return
			}
			if {[string is integer -strict $skipRows] != 1 || $skipRows < 0} {
				eval $close_progression_bar
				tk_messageBox -parent .w_PLT_ImportData -type ok -icon error -title "Error!"\
					-message "\"Skip Rows\" must be a non-negative integer."
				return
			}
			
			# Data is named after the file name. If the name already exists then "_i", where i is an integer, is appended.
			set dataName [file tail $importFile]
			if {[lsearch -exact $DataNames $dataName] != -1} {
				set i 1
				while {[lsearch -exact $DataNames "${dataName}_$i"] != -1} {
					incr i
				}
				set dataName "${dataName}_$i"
			}
			
			# Set the data separator
			set sep [sep_char $separatorSelect]
		
			# Import data in Scilab and create variable names.
			if {$varNamesFromRowYesNo == 1} {
				ScilabEval "PLT_Macro_importData('$dataName', '$importFile', $skipRows, $sep, $varNameRow);" "seq"
			} else {
				ScilabEval "PLT_Macro_importData('$dataName', '$importFile', $skipRows, $sep, -1);" "seq"
			}	
		}
		
		eval $close_progression_bar
		destroy .w_PLT_ImportData
	}
	
	#***********************************************************************
	# cb_btn_Cancel -- callback to the "cancel" button
	#***********************************************************************
	proc cb_btn_Cancel {} {
		destroy .w_PLT_ImportData
	}
}