# This file is part of QuickPlot

#***********************************************************************
# Import Variable Names Dialog Box Module
#
#     Construct a dialog box for importing variable names from file
#
# Public Procedures:
#     dbox_ImportVarNames -- create a dialog box for importing variable names from a file
#
# Namespace:
#     ns_ImportVarNames
#
# Private Procedures:
#     cb_btn_getOpen_ImportFile -- callback to the "open file" button
#     cb_btn_Import -- callback to the "import" button
#     cb_btn_Cancel -- callback to the "cancel" button
#***********************************************************************

#***********************************************************************
# dbox_ImportVarNames -- create a dialog box for importing variable names from a file
#***********************************************************************
proc dbox_ImportVarNames {} {

	global DataNames; # list of all data names
	global SelectedDataIndices; # list of indices of currently selected data
	
	# Check if data exist
	if {[llength $DataNames] < 1} {
		tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
			-message "Data must exist before variable names are imported."
		return
	}
	
	# Reset some of the namespace variables
	set ::ns_ImportVarNames::importFile ""; # path of the file from which data is imported

	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
	# Import Variable Names Dialog Box
	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

	# Create a new top-level window for the dialog box
	toplevel .w_PLT_ImportVarNames
	wm title .w_PLT_ImportVarNames "Import Variable Names from File"
	wm iconbitmap .w_PLT_ImportVarNames $::ICON_FILE

	# In the dialog box, create "Setting" and "Buttons" frames
	set ::ns_ImportVarNames::frameSetting [set frameSetting [frame .w_PLT_ImportVarNames.f_Setting]]
	set ::ns_ImportVarNames::frameButtons [set frameButtons [frame .w_PLT_ImportVarNames.f_Buttons]]
	grid $frameSetting -padx 10 -pady 10 -sticky news
	grid $frameButtons -padx 10 -pady 10 -sticky news
	grid rowconfigure .w_PLT_ImportVarNames 0 -weight 1
	grid columnconfigure .w_PLT_ImportVarNames 0 -weight 1

	# In Setting frame:
	label $frameSetting.lab_DataName -text "Select data to which variable names are imported" -anchor w
	frame $frameSetting.f_DataSelect; # Data-Select frame
	label $frameSetting.lab_ImportFile -text "Variable name file" -anchor w
	entry $frameSetting.ent_ImportFile -textvariable ::ns_ImportVarNames::importFile -state readonly
	button $frameSetting.btn_getOpen_ImportFile -text "Browse" -command ::ns_ImportVarNames::cb_btn_getOpen_ImportFile
	grid $frameSetting.lab_DataName -padx 4 -pady 2 -sticky ew
	grid $frameSetting.f_DataSelect -padx 4 -pady 2 -sticky news
	grid $frameSetting.lab_ImportFile -padx 4 -pady 2 -sticky ew
	grid $frameSetting.ent_ImportFile $frameSetting.btn_getOpen_ImportFile -padx 4 -pady 2 -sticky ew
	grid rowconfigure $frameSetting 1 -weight 1
	grid columnconfigure $frameSetting 0 -weight 1
	
	# In Data-Select frame
	listbox $frameSetting.f_DataSelect.lsb_DataSelect -height 7\
		-xscrollcommand {$frameSetting.f_DataSelect.scrl_DataSelect_h set}\
		-yscrollcommand {$frameSetting.f_DataSelect.scrl_DataSelect_v set}\
		-selectmode extended -listvariable DataNames -activestyle none
	scrollbar $frameSetting.f_DataSelect.scrl_DataSelect_h -orient horizontal\
		-command "$frameSetting.f_DataSelect.lsb_DataSelect xview"
	scrollbar $frameSetting.f_DataSelect.scrl_DataSelect_v -orient vertical\
		-command "$frameSetting.f_DataSelect.lsb_DataSelect yview"
	grid $frameSetting.f_DataSelect.lsb_DataSelect $frameSetting.f_DataSelect.scrl_DataSelect_v -sticky news
	grid $frameSetting.f_DataSelect.scrl_DataSelect_h -sticky news
	grid rowconfigure $frameSetting.f_DataSelect 0 -weight 1
	grid columnconfigure $frameSetting.f_DataSelect 0 -weight 1
	$frameSetting.f_DataSelect.lsb_DataSelect configure\
		-xscrollcommand [list sbset $frameSetting.f_DataSelect.scrl_DataSelect_h]\
		-yscrollcommand [list sbset $frameSetting.f_DataSelect.scrl_DataSelect_v]
	
	# In Buttons frame:
	button $frameButtons.btn_Import -text Import -width 15 -command ::ns_ImportVarNames::cb_btn_Import
	button $frameButtons.btn_Cancel -text Cancel -width 15 -command ::ns_ImportVarNames::cb_btn_Cancel
	pack $frameButtons.btn_Cancel $frameButtons.btn_Import -padx 10 -side right
	
	# Set grab to this window
	grab set .w_PLT_ImportVarNames
}

#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
# Import Variable Names Namespace
#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
namespace eval ns_ImportVarNames {

	# Namespace variables
	variable frameSetting
	variable frameButtons
	variable importFile ""; # path of the file from which data is imported
	
	#***********************************************************************
	# cb_btn_getOpen_ImportFile -- callback to the "open file" button
	#***********************************************************************
	proc cb_btn_getOpen_ImportFile {} {
		global CurrentDir; # current directory path
		variable importFile; # path of the file from which data is imported
		
		set typelist {
			{"All Files" {*}}
			{"Text Files" {".txt"}}
		}
		set filePath [tk_getOpenFile -initialdir $CurrentDir -parent .w_PLT_ImportVarNames -filetypes $typelist]
		if {$filePath ne ""} {
			set importFile $filePath
		}
	}
	
	#***********************************************************************
	# cb_btn_Import -- callback to the "import" button
	#***********************************************************************
	proc cb_btn_Import {} {
		global VarNamesList
		
		variable frameSetting
		variable importFile; # path of the file from which data is imported
		
		set dataIndices [$frameSetting.f_DataSelect.lsb_DataSelect curselection]
		
		if {$dataIndices == ""} {
			tk_messageBox -parent .w_PLT_ImportVarNames -type ok -icon error -title "Error!"\
				-message "Please select data to which variable names are imported."
			return
		}
		
		set nCol ""
		foreach dataIndex $dataIndices {
			if {$nCol == ""} {
				set nCol [llength [lindex $VarNamesList $dataIndex]]
			} else {
				if {[llength [lindex $VarNamesList $dataIndex]] != $nCol} {
					tk_messageBox -parent .w_PLT_ImportVarNames -type ok -icon error -title "Error!"\
						-message "Selected data must have the same number of columns."
					return
				}
			}
		}
		
		if {[file exists $importFile] != 1} {
			tk_messageBox -parent .w_PLT_ImportVarNames -type ok -icon error -title "Error!"\
				-message "File \"$importFile\" doesn't exist or isn't accessible."
			return
		}
		
		foreach dataIndex $dataIndices {
			# Import variable names in Scilab.
			ScilabEval "PLT_Macro_importVarNames($dataIndex, '$importFile');" "seq"
		}
		
		destroy .w_PLT_ImportVarNames
	}
	
	#***********************************************************************
	# cb_btn_Cancel -- callback to the "cancel" button
	#***********************************************************************
	proc cb_btn_Cancel {} {
		destroy .w_PLT_ImportVarNames
	}
}