# This file is part of QuickPlot

#***********************************************************************
# Permute Data Dialog Box Module
#
#     Construct a dialog box for permuting data
#
# Public Procedures:
#     dbox_PermuteData -- create a dialog box for permuting data
#
# Namespace:
#     ns_PermuteData
#
# Private Procedures:
#     cb_btn_MoveUp -- callback to the "MoveUp" button
#     cb_btn_MoveDown -- callback to the "MoveDown" button
#     cb_btn_OK -- callback to the "OK" button
#     cb_btn_Cancel -- callback to the "Cancel" button
#***********************************************************************

#***********************************************************************
#  dbox_PermuteData -- create a dialog box for permuting data
#***********************************************************************
proc dbox_PermuteData {} {

	global DataNames; # list of all data names
	global VarNamesList; # list of lists of variable names
	global SelectedDataIndices; # list of indices of currently selected data
	
	# Check if data exist
	if {[llength $DataNames] < 1} {
		tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
			-message "There is no data."
		return
	}
	
	# Initialize some of the namespace variables
	set ::ns_PermuteData::dataNames $DataNames;
	set ::ns_PermuteData::varNamesList $VarNamesList;
	set ::ns_PermuteData::selectedDataIndices $SelectedDataIndices;

	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
	# Permute Data Dialog Box
	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

	# Create a new top-level window for the dialog box
	toplevel .w_PLT_PermuteData
	wm title .w_PLT_PermuteData "Permute Data"
	wm iconbitmap .w_PLT_PermuteData $::ICON_FILE

	# In the dialog box, create "DataList", "Control" and "Buttons" frames
	set ::ns_PermuteData::frameDataList [set frameDataList [frame .w_PLT_PermuteData.f_DataList]]
	set ::ns_PermuteData::frameControl [set frameControl [frame .w_PLT_PermuteData.f_Control]]
	set ::ns_PermuteData::frameButtons [set frameButtons [frame .w_PLT_PermuteData.f_Buttons]]
	grid $frameDataList $frameControl -padx 10 -pady 10 -sticky news
	grid $frameButtons - -padx 10 -pady 10 -sticky news
	grid columnconfigure .w_PLT_PermuteData 0 -weight 1
	grid rowconfigure .w_PLT_PermuteData 0 -weight 1

	# In DataList frame:
	label $frameDataList.lab_Instruction -text "Select data to move up or down" -anchor w
	listbox $frameDataList.lsb_Data -height 7\
		-xscrollcommand {$frameDataList.scrl_Data_h set}\
		-yscrollcommand {$frameDataList.scrl_Data_v set}\
		-selectmode single -selectbackground blue -selectforeground white -listvariable ::ns_PermuteData::dataNames -activestyle none
	bind $frameDataList.lsb_Data <ButtonPress-1> {::ns_PermuteData::cb_select_data_press}
	bind $frameDataList.lsb_Data <ButtonRelease-1> {::ns_PermuteData::cb_select_data_release}
	scrollbar $frameDataList.scrl_Data_h -orient horizontal\
		-command "$frameDataList.lsb_Data xview"
	scrollbar $frameDataList.scrl_Data_v -orient vertical\
		-command "$frameDataList.lsb_Data yview"
	grid $frameDataList.lab_Instruction -sticky w
	grid $frameDataList.lsb_Data $frameDataList.scrl_Data_v -sticky news
	grid $frameDataList.scrl_Data_h -sticky news
	grid columnconfigure $frameDataList 0 -weight 1
	grid rowconfigure $frameDataList 1 -weight 1
	$frameDataList.lsb_Data configure\
		-xscrollcommand [list sbset $frameDataList.scrl_Data_h]\
		-yscrollcommand [list sbset $frameDataList.scrl_Data_v]
		
	# In Control frame:
	button $frameControl.btn_MoveUp -text {Move Up} -width 10 -command ::ns_PermuteData::cb_btn_MoveUp
	button $frameControl.btn_MoveDown -text {Move Down} -width 10 -command ::ns_PermuteData::cb_btn_MoveDown
	pack $frameControl.btn_MoveUp -padx 5 -side top -anchor s -expand 1
	pack $frameControl.btn_MoveDown -padx 5 -side top -anchor n -expand 1
	
	# In Buttons frame:
	button $frameButtons.btn_OK -text OK -width 15 -command ::ns_PermuteData::cb_btn_OK
	pack $frameButtons.btn_OK -padx 10 -side right -expand 1
	
	# Set grab to this window
	grab set .w_PLT_PermuteData
}

#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
# Permute Data Namespace
#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
namespace eval ns_PermuteData {

	# Namespace variables
	variable frameDataList
	variable frameControl
	variable frameButtons
	variable dataNames ""; # list of all data names
	variable varNamesList ""; # list of lists of variable names
	variable selectedDataIndices ""; # list of indices of currently selected data
	variable dataIndex ""; # selected data index in the listbox
	
	#***********************************************************************
	# cb_select_data_press -- callback when the data list box is mouse-pressed
	#***********************************************************************
	proc cb_select_data_press {} {
		::ns_PermuteData::mac_decolor_data_listbox
	}

	#***********************************************************************
	# cb_select_data_release -- callback when the data list box is mouse-released
	#***********************************************************************
	proc cb_select_data_release {} {
		::ns_PermuteData::mac_get_dataIndex
		::ns_PermuteData::mac_paint_data_listbox
	}
	
	#***********************************************************************
	# mac_decolor_data_listbox -- decolor all items in the data list box
	#***********************************************************************
	proc mac_decolor_data_listbox {} {
		variable frameDataList
		variable dataNames; # list of all data names
		
		for {set i 0} {$i < [llength $dataNames]} {incr i} {
			$frameDataList.lsb_Data itemconfigure $i -background white -foreground black
		}
	}

	#***********************************************************************
	# mac_get_dataIndex -- get "dataIndex" from the data list box
	#***********************************************************************
	proc mac_get_dataIndex {} {
		variable frameDataList
		variable dataIndex; # selected data index in the listbox

		set dataIndex [$frameDataList.lsb_Data curselection]
	}

	#***********************************************************************
	# mac_paint_data_listbox -- paint the selected items in the data list box
	#***********************************************************************
	proc mac_paint_data_listbox {} {
		variable frameDataList
		variable dataNames; # list of all data names
		variable dataIndex; # selected data index in the listbox
		
		$frameDataList.lsb_Data itemconfigure $dataIndex -background blue -foreground white
	}

	#***********************************************************************
	#  cb_btn_MoveUp -- callback to the "MoveUp" button
	#***********************************************************************
	proc cb_btn_MoveUp {} {
		variable frameDataList
		variable dataNames; # list of all data names
		variable varNamesList; # list of lists of variable names
		variable selectedDataIndices; # list of indices of currently selected data
		variable dataIndex; # selected data index in the listbox
		
		if {$dataIndex eq "" || $dataIndex == 0} {
			return
		}
		
		set newDataIndex [expr {$dataIndex - 1}]
		
		ScilabEval "PLT_Macro_permuteData($dataIndex, $newDataIndex);" "seq"
		
		set dataName [lindex $dataNames $dataIndex]
		set dataNames [lreplace $dataNames $dataIndex $dataIndex]
		set dataNames [linsert $dataNames $newDataIndex $dataName]
		
		set varNames [lindex $varNamesList $dataIndex]
		set varNamesList [lreplace $varNamesList $dataIndex $dataIndex]
		set varNamesList [linsert $varNamesList $newDataIndex $varNames]
		
		for {set i 0} {$i < [llength $selectedDataIndices]} {incr i} {
			if {[lindex $selectedDataIndices $i] == $dataIndex} {
				lset selectedDataIndices $i $newDataIndex
			} elseif {[lindex $selectedDataIndices $i] == $newDataIndex} {
				lset selectedDataIndices $i $dataIndex
			}
		}
		
		set selectedDataIndices [lsort -integer -increasing $selectedDataIndices]
		
		set dataIndex $newDataIndex
		
		$frameDataList.lsb_Data selection clear 0 end
		::ns_PermuteData::mac_decolor_data_listbox
		::ns_PermuteData::mac_paint_data_listbox
	}
	
	#***********************************************************************
	#  cb_btn_MoveDown -- callback to the "MoveDown" button
	#***********************************************************************
	proc cb_btn_MoveDown {} {
		
		variable frameDataList
		variable dataNames; # list of all data names
		variable varNamesList; # list of lists of variable names
		variable selectedDataIndices; # list of indices of currently selected data
		variable dataIndex; # selected data index in the listbox
		
		if {$dataIndex eq "" || $dataIndex == [expr {[llength $dataNames]-1}]} {
			return
		}
		
		set newDataIndex [expr {$dataIndex + 1}]
		
		ScilabEval "PLT_Macro_permuteData($dataIndex, $newDataIndex);" "seq"
		
		set dataName [lindex $dataNames $dataIndex]
		set dataNames [lreplace $dataNames $dataIndex $dataIndex]
		set dataNames [linsert $dataNames $newDataIndex $dataName]
		
		set varNames [lindex $varNamesList $dataIndex]
		set varNamesList [lreplace $varNamesList $dataIndex $dataIndex]
		set varNamesList [linsert $varNamesList $newDataIndex $varNames]
		
		for {set i 0} {$i < [llength $selectedDataIndices]} {incr i} {
			if {[lindex $selectedDataIndices $i] == $dataIndex} {
				lset selectedDataIndices $i $newDataIndex
			} elseif {[lindex $selectedDataIndices $i] == $newDataIndex} {
				lset selectedDataIndices $i $dataIndex
			}
		}
		
		set selectedDataIndices [lsort -integer -increasing $selectedDataIndices]
		
		set dataIndex $newDataIndex
		
		$frameDataList.lsb_Data selection clear 0 end
		::ns_PermuteData::mac_decolor_data_listbox
		::ns_PermuteData::mac_paint_data_listbox
	}
	
	#***********************************************************************
	#  cb_btn_OK -- callback to the "OK" button
	#***********************************************************************
	proc cb_btn_OK {} {
		global DataNames; # list of all data names
		global VarNamesList; # list of lists of variable names
		global SelectedDataIndices; # list of indices of currently selected data
		
		variable dataNames; # list of all data names
		variable varNamesList; # list of lists of variable names
		variable selectedDataIndices; # list of indices of currently selected data
		
		set DataNames $dataNames
		set VarNamesList $varNamesList
		set SelectedDataIndices $selectedDataIndices
		
		cb_filter_data
		
		destroy .w_PLT_PermuteData
	}
}