# This file is part of QuickPlot

#***********************************************************************
# Rename Data Dialog Box Module
#
#     Construct a dialog box for renaming data
#
# Public Procedures:
#     dbox_RenameData -- create a dialog box for renaming data
#
# Namespace:
#     ns_RenameData
#
# Private Procedures:
#     cb_btn_Apply -- callback to the "Apply" button
#     cb_btn_OK -- callback to the "OK" button
#     cb_btn_Cancel -- callback to the "Cancel" button
#***********************************************************************

#***********************************************************************
# dbox_RenameData -- create a dialog box for renaming data
#***********************************************************************
proc dbox_RenameData {} {

	global DataNames; # list of all data names
	global SelectedDataIndices; # list of indices of currently selected data
	
	# Check if data exist
	if {[llength $DataNames] < 1} {
		tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
			-message "There is no data."
		return
	}
	
	# Reset some of the namespace variables
	if {$SelectedDataIndices eq ""} {
		set ::ns_RenameData::dataName [lindex $DataNames 0]
	} else {
		set ::ns_RenameData::dataName [lindex $DataNames [lindex $SelectedDataIndices 0]]
	}
	set ::ns_RenameData::newDataName {}

	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
	# Rename Data Dialog Box
	#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
	
	# Create a new top-level window for the dialog box
	toplevel .w_PLT_RenameData
	wm title .w_PLT_RenameData "Rename Data"
	wm iconbitmap .w_PLT_RenameData $::ICON_FILE
	
	# In the dialog box, create "Setting" and "Buttons" frames
	set ::ns_RenameData::frameSetting [set frameSetting [frame .w_PLT_RenameData.f_Setting]]
	set ::ns_RenameData::frameButtons [set frameButtons [frame .w_PLT_RenameData.f_Buttons]]
	grid $frameSetting -padx 10 -pady 10 -sticky ew
	grid $frameButtons -padx 10 -pady 10 -sticky ew
	grid columnconfigure .w_PLT_RenameData 0 -weight 1

	# In Setting frame:
	label $frameSetting.lab_DataName -text "Data" -anchor w
	ttk::combobox $frameSetting.cmb_DataName -textvariable ::ns_RenameData::dataName -values $DataNames -state readonly
	label $frameSetting.lab_DataNameNew -text "New data name" -anchor w
	entry $frameSetting.ent_DataNameNew -textvariable ::ns_RenameData::newDataName
	grid $frameSetting.lab_DataName $frameSetting.cmb_DataName -padx 4 -pady 2 -sticky ew
	grid $frameSetting.lab_DataNameNew $frameSetting.ent_DataNameNew -padx 4 -pady 2 -sticky ew
	grid columnconfigure $frameSetting 1 -weight 1

	# In Buttons frame:
	button $frameButtons.btn_Apply -text Apply -width 15 -command ::ns_RenameData::cb_btn_Apply
	button $frameButtons.btn_OK -text OK -width 15 -command ::ns_RenameData::cb_btn_OK
	button $frameButtons.btn_Cancel -text Cancel -width 15 -command ::ns_RenameData::cb_btn_Cancel
	pack $frameButtons.btn_Cancel $frameButtons.btn_Apply $frameButtons.btn_OK  -padx 10 -side right
	
	# Set grab to this window
	grab set .w_PLT_RenameData
}

#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
# Rename Data Namespace
#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
namespace eval ns_RenameData {

	# Namespace variables
	variable frameSetting
	variable frameButtons
	variable dataName
	variable newDataName;
	
	#***********************************************************************
	# cb_btn_Apply -- callback to the "Apply" button
	#***********************************************************************
	proc cb_btn_Apply {} {
		global DataNames; # list of all data names

		variable frameSetting
		variable newDataName; # new data name
		
		set newDataName [string trim $newDataName]
		
		if {$newDataName == ""} {
			tk_messageBox -parent .w_PLT_RenameData -type ok -icon error -title "Error!"\
				-message "\"New Data Name\" cannot be an empty string"
			error {Failed to rename data}
		}
		
		set dataIndex [$frameSetting.cmb_DataName current]
				
		if {[lsearch -exact [lreplace $DataNames $dataIndex $dataIndex] $newDataName] != -1} {
			tk_messageBox -parent .w_PLT_RenameData -type ok -icon error -title "Error!"\
				-message "$newDataName already exists. Please choose a different data name."
			error {Failed to rename data}
		}
		
		ScilabEval "PLT_Macro_renameData($dataIndex, '$newDataName');" "seq"
		
		lset DataNames $dataIndex $newDataName
		$frameSetting.cmb_DataName config -values $DataNames
		set ::ns_RenameData::dataName $newDataName
	}
	
	#***********************************************************************
	# cb_btn_OK -- callback to the "OK" button
	#***********************************************************************
	proc cb_btn_OK {} {
		::ns_RenameData::cb_btn_Apply
		destroy .w_PLT_RenameData
	}
	
	#***********************************************************************
	# cb_btn_Cancel -- callback to the "Cancel" button
	#***********************************************************************
	proc cb_btn_Cancel {} {
		destroy .w_PLT_RenameData
	}
}