# This file is part of QuickPlot

#***********************************************************************
# Functions Module
#
#     Common functions for QuickPlot
#
# Public Procedures:
#     sep_char -- given a separator choice, return a corresponding Scilab character vector
#     sbset -- make scrollbars to appear and disappear according to whether they are needed
#     init_list -- create a list of an arbitrary length with a specified value repeated
#     minlength_lists -- return the minimum length of lists
#     common_elements -- find common elements from multiple lists
#     is_included -- check if an element is included in a list
#     list2strvec -- convert a list to a string that can be used to define a string vector in Scilab
#     filter_list -- filter a list using Glob matching
#     filter_list_indices -- filter a list using Glob matching (returns indices)
#     read_file -- read lines from file
#     invoke_browser -- call an external browser to display a page given a url
#***********************************************************************

#***********************************************************************
#  sep_char -- given a separator choice, return a corresponding Scilab character vector 
#
#  Parameter:
#    separatorSelect -- separator choice from SEPARATOR_LIST
#***********************************************************************
set SEPARATOR_LIST {"space/tab" "comma"}
proc sep_char {separatorSelect} {
	switch -- $separatorSelect {
		"space/tab" {set sep {[' ', ascii(9)]}}
		"comma" {set sep {','}}
		default {error "Error! Unknown separator."}
	}
	return $sep
}

#***********************************************************************
#  sbset -- make scrollbars to appear and disappear according to whether they are needed
#
#  Usage:
#    $scrollee configure -xscrollcommand [list sbset $hsb] -yscrollcommand [list sbset $vsb]
#
#  Reference:
#    http://wiki.tcl.tk/950
#***********************************************************************
proc sbset {sb first last} {
	if {$first <= 0 && $last >= 1} {
		grid remove $sb
	} else {
		grid $sb
	}
	$sb set $first $last
}

#***********************************************************************
# init_list -- create a list of an arbitrary length with a specified
#              value repeated
#
# Parameter
#     value -- element value
#     n -- length of the list to be created
#     
# Return
#     L -- list created
#
# Limitation
#     No space is allowed in the value
#***********************************************************************
proc init_list {value n} {
	set L [split [string repeat "$value " $n] { }]
	set L [lreplace $L end end]; # remove the redundant empty element
	return $L
}

#***********************************************************************
# minlength_lists -- return the minimum length of lists
#
# Parameter
#     list_of_lists -- list of lists
#     
# Return
#     n -- minimum length of lists
#***********************************************************************
proc minlength_lists {list_of_lists} {
	# Initialize n to be the length of the first list.
	set n [llength [lindex $list_of_lists 0]]
	# For each list obtain length and update n with the minimum length.
	foreach curlist $list_of_lists {
		set n [expr {min($n,[llength $curlist])}]
	}
	return $n
}

#***********************************************************************
# common_elements -- find common elements from multiple lists
#
# Parameter
#     list_of_lists -- list of lists
#     
# Return
#     ans -- list of common elements
#***********************************************************************
proc common_elements {list_of_lists} {
	set firstlist [lindex $list_of_lists 0]
	set n_lists [llength $list_of_lists]
	if {$n_lists == 1} {
		set ans $firstlist
	} else {
		set ans {}
		foreach elem $firstlist {
			for {set i 1} {$i < $n_lists} {incr i} {
				if {[lsearch -exact [lindex $list_of_lists $i] $elem] != -1} {
					if {$i == [expr {$n_lists-1}]} {
						lappend ans $elem
					} else {
						continue
					}
				} else {
					break
				}
			}
		}
	}
	return $ans
}

#***********************************************************************
# is_included -- check if an element is included in a list
#
# Parameter
#     element
#     list
#     
# Return
#     ans -- 1: element is included in list, 0: otherwise
#***********************************************************************
proc is_included {element list} {
	if {[lsearch -exact $list $element] == -1} {
		set ans 0; # false
	} else {
		set ans 1; # true
	}
}

#***********************************************************************
# list2strvec -- convert a list to a string that can be used to define
#                a string vector in Scilab
#
# Parameter
#     in_list -- a list (e.g. {abc de fghi})
#
# Return
#     out_strvec -- a string that can be used to define a string vector
#                   in Scilab (e.g. ['abc'; 'de'; 'fghi'; ]
#***********************************************************************
proc list2strvec {in_list} {
	set out_strvec {[}
	foreach element $in_list {
		append out_strvec "'$element'; "
	}
	append out_strvec {]}
	return $out_strvec
}

#***********************************************************************
# filter_list -- filter a list using Glob matching
#
# Parameter
#     in_list -- input list
#     pattern -- filter pattern
#
# Return
#     filtered_list -- filtered list
#***********************************************************************
proc filter_list {in_list pattern} {
	# If pattern is empty then no filtering is carried out
	if {$pattern == ""} {
		set pattern {*}; # star is a wild card
	}
	# Use lsearch command with Glob matching for filtering
	set filtered_list [lsearch -all -inline $in_list $pattern]
	return $filtered_list
}

#***********************************************************************
# filter_list_indices -- filter a list using Glob matching (returns indices)
#
# Parameter
#     in_list -- input list
#     pattern -- filter pattern
#
# Return
#     filtered_indices -- indices of filtered list elements
#***********************************************************************
proc filter_list_indices {in_list pattern} {
	# If pattern is empty then no filtering is carried out
	if {$pattern == ""} {
		set pattern {*}; # star is a wild card
	}
	# Use lsearch command with Glob matching for filtering
	set filtered_indices [lsearch -all $in_list $pattern]
	return $filtered_indices
}

#***********************************************************************
# read_file -- read lines from file
#
# Parameter
#     filename -- path of the file to read
#
# Return
#     lines -- a list of lines read from file
#***********************************************************************
proc read_file {filename} {
	set lines ""
	set fID [open $filename]
	while {[gets $fID line] >= 0} {
		lappend lines $line
	}
	close $fID
	return $lines
}

#***********************************************************************
# invoke_browser -- call an external browser to display a page given a url
#
# Parameter
#     url -- url of a page
#
# Reference
#     https://wiki.tcl-lang.org/page/Invoking+browsers
#***********************************************************************
proc invoke_browser {url} {
	# open is the OS X equivalent to xdg-open on Linux, start is used on Windows
	set commands {xdg-open open start}
	foreach browser $commands {
		if {$browser eq "start"} {
			set command [list {*}[auto_execok start] {}]
		} else {
			set command [auto_execok $browser]
		}
		if {[string length $command]} {
			break
		}
	}

	if {[string length $command] == 0} {
		return -code error "couldn't find browser"
	}
	if {[catch {exec {*}$command $url &} error]} {
		return -code error "couldn't execute '$command': $error"
	}
}