# This file is part of QuickPlot

#***********************************************************************
# Macros Module
#
#     Macros for QuickPlot
#
# Public Procedures:
#     mac_delete_all_data -- delete all data
#     mac_clear -- delete all data and close all figures
#     mac_plot_data -- plot selected data in a figure
#     mac_clear_figure -- clear the current figure
#     mac_new_figure -- create a new figure
#     mac_set_current_fig -- select a figure to the current figure
#     mac_get_current_fig_id -- get the current figure ID -> CurrentFigID
#     mac_close_all -- close all figures
#     mac_set_properties -- apply properties
#     mac_show_graphic_editor -- open the graphic editor for the current axes
#     mac_edit_legend -- open a dialog box to edit legend texts in the current figure
#     mac_twinkle_curve -- open a dialog box to twinkle a curve in the current axes
#     mac_delete_curve -- open a dialog box to delete a curve in the current axes
#     mac_export_curve_to_workspace -- open a dialog box to export curve data to workspace
#     mac_exit_program -- clean up and exit program
#     mac_exit_scilab -- exit Scilab
#     mac_filter_data -- filter 'data'
#     mac_get_SelectedDataIndices -- get "SelectedDataIndices" from the data list-box
#     mac_filter_SelectedDataIndices -- filter "SelectedDataIndices"
#     mac_decolor_data_listbox -- decolor all items in the data list-box
#     mac_paint_data_listbox -- paint the selected items in the data list-box
#     mac_update_XVar -- update "FilteredXVars"
#     mac_get_SelectedXVars -- get "SelectedXVars" from the x variable list-box
#     mac_filter_SelectedXVars -- filter "SelectedXVars"
#     mac_update_YVar -- update "FilteredYVars"
#     mac_get_SelectedYVars -- get "SelectedYVars" from the y variable list-box
#     mac_filter_SelectedYVars -- filter "SelectedYVars"
#     mac_decolor_xvar_listbox -- decolor all items in the x variable list-box
#     mac_paint_xvar_listbox -- paint the selected items in the x variable list-box
#     mac_decolor_yvar_listbox -- decolor all items in the y variable list-box
#     mac_paint_yvar_listbox -- paint the selected items in the y variable list-box
#     mac_import_datafilter -- import preset data filter
#     mac_import_varfilter -- import preset variable filter
#     mac_change_curdirectory -- change current directory
#     mac_show_curdirectory -- show current directory
#     mac_about_software -- show information about this software
#     mac_open_manual -- open user's manual with external browser
#***********************************************************************

#***********************************************************************
# mac_delete_all_data -- delete all data
#***********************************************************************
proc mac_delete_all_data {} {

		global DataNames; # list of all data names
		global VarNamesList; # list of lists of variable names
		global FilteredXVars; # filtered list of x variables
		global FilteredYVars; # filtered list of y variables
		global SelectedDataIndices; # list of indices of currently selected data
		global SelectedXVars; # list of currently selected x variables
		global SelectedYVars; # list of currently selected y variables
		
		for {set i [expr {[llength $DataNames]-1}]} {$i >= 0} {incr i -1} {
			ScilabEval "PLT_Macro_deleteData($i);" "seq"
		}
		
		set DataNames {}
		set VarNamesList {}
			
		cb_filter_data
		cb_ckb_Browse
		set FilteredXVars {}
		set FilteredYVars {}
		
		set SelectedDataIndices {}
		set SelectedXVars {}
		set SelectedYVars {}
}

#***********************************************************************
# mac_clear -- delete all data and close all figures
#***********************************************************************
proc mac_clear {} {

	if {[tk_messageBox -parent .w_PLT -type okcancel -icon question -title "Confirmation"\
			-message "Are you sure you want to clear data and close all figures?"] eq "ok"} {

		# Delete all data
		mac_delete_all_data
		
		# Close all figures
		mac_close_all
	}
}

#***********************************************************************
# mac_plot_data -- plot selected data in a figure
#
# Parameter
#     newPlotYesNo -- 1: data is plotted in a new figure, 
#                     0: plot is added in the current figure if there is
#                        an open figure, otherwise a new figure is created
#***********************************************************************
proc mac_plot_data {newPlotYesNo} {

	global VarNamesList; # list of lists of variable names
	global SelectedDataIndices; # list of indices of currently selected data
	global SelectedXVars; # list of currently selected x variables
	global SelectedYVars; # list of currently selected y variables
	global ApplyPropAlwaysYesNo; # flag: whether if the graph property is applied always or not
	global XVarYesNo; # flag: whether to use x variable or not
	
	# Error if no data is selected
	if {$SelectedDataIndices eq ""} {
		tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
			-message "Please select data."
		return
	}
	# Error if x variable is to be used and no x variable is selected
	if {$XVarYesNo == 1} {
		if {$SelectedXVars eq ""} {
			tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
				-message "Please select x variable."
			return
		}
	}
	# Error if no y variable is selected
	if {$SelectedYVars eq ""} {
		tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
			-message "Please select y variable."
		return
	}
	# Plot data
	foreach dataindex $SelectedDataIndices {
		set yvarindices {}
		foreach yvar $SelectedYVars {
			lappend yvarindices [lsearch -exact [lindex $VarNamesList $dataindex] $yvar]
		}
		if {$XVarYesNo == 1} {
			set xvarindex [lsearch -exact [lindex $VarNamesList $dataindex] [lindex $SelectedXVars 0]]
			ScilabEval "PLT_Macro_plotData(\[$dataindex\], \[$xvarindex\], \[$yvarindices\], $newPlotYesNo);" "seq"
		} else {
			ScilabEval "PLT_Macro_plotData(\[$dataindex\], \[\], \[$yvarindices\], $newPlotYesNo);" "seq"
		}
	}
	# Apply the user specified properties if this option is selected
	if {$ApplyPropAlwaysYesNo == 1} {
		mac_set_properties
	}
	
	ScilabEval "drawnow();" "seq"
	mac_get_current_fig_id
}

#***********************************************************************
# mac_clear_figure -- clear the current figure
#***********************************************************************
proc mac_clear_figure {} {

	ScilabEval "PLT_Macro_clearFigure();" "seq"
	mac_get_current_fig_id
}

#***********************************************************************
# mac_new_figure -- create a new figure
#***********************************************************************
proc mac_new_figure {} {

	ScilabEval "\
		h=scf(); clf(h);\
		h.figure_name = 'Figure No.%d';\
		h.event_handler = 'PLT_CB_handleFigureEvent';\
		h.event_handler_enable = 'on';\
		clear h;\
	" "seq"
	mac_get_current_fig_id
}

#***********************************************************************
# mac_set_current_fig -- select a figure to the current figure
#
# Parameter:
#     figID -- figure ID
#***********************************************************************
proc mac_set_current_fig {figID} {

	ScilabEval "PLT_Macro_setCurrentFig($figID);" "seq"
	mac_get_current_fig_id
}

#***********************************************************************
# mac_get_current_fig_id -- get the current figure ID -> CurrentFigID
#***********************************************************************
proc mac_get_current_fig_id {} {

	ScilabEval {\
		TCL_SetVar('CurrentFigID', string(PLT_Macro_getCurrentFigId()));\
		TCL_SetVar('ListFigIDs', PLT_Util_strvec2list(string(winsid())));\
		TCL_EvalStr('::$framePlotButtons.cmb_CurrentFigID configure -value $ListFigIDs');\
	} "seq"
}

#***********************************************************************
# mac_close_all -- close all figures
#***********************************************************************
proc mac_close_all {} {

	ScilabEval "PLT_Macro_closeAll();" "seq"
	mac_get_current_fig_id
}

#***********************************************************************
# mac_set_properties -- apply properties
#***********************************************************************
proc mac_set_properties {} {

	global PlotTitle; # title string
	global PlotXLabel; # x label string
	global PlotYLabel; # y label string
	global PlotFontSize; # font size for the plot
	global PlotLineYesNo; # flag: whether to show lines or not
	global PlotLineThickness; # line thickness
	global PlotMarkYesNo; # flag: whether to show marks or not
	global PlotMarkSize; # mark size
	global PlotGridYesNo; # flag: whether to show grids or not
	global PlotGridColor; # grid color
	global PlotLegendYesNo; # flag: whether to show legend or not
	global PlotLegendFontSize; # legend font size
	global PlotLegendLocation; # legend location
	global PlotXLimYesNo; # flag: whether to use x limits or not
	global PlotYLimYesNo; # flag: whether to use y limits or not
	global PlotXMin PlotXMax PlotYMin PlotYMax; # axes limits
	
	ScilabEval "PLT_Macro_setProperties(\
		struct(\
			'title', '$PlotTitle',\
			'xLabel', '$PlotXLabel',\
			'yLabel', '$PlotYLabel',\
			'fontSize', $PlotFontSize,\
			'lineYesNo', $PlotLineYesNo,\
			'lineThickness', $PlotLineThickness,\
			'markYesNo', $PlotMarkYesNo,\
			'markSize', $PlotMarkSize,\
			'gridYesNo', $PlotGridYesNo,\
			'gridColor', $PlotGridColor,\
			'legendYesNo', $PlotLegendYesNo,\
			'legendFontSize', $PlotLegendFontSize,\
			'legendLocation', '$PlotLegendLocation',\
			'xLimYesNo', $PlotXLimYesNo,\
			'xMin', $PlotXMin,\
			'xMax', $PlotXMax,\
			'yLimYesNo', $PlotYLimYesNo,\
			'yMin', $PlotYMin,\
			'yMax', $PlotYMax\
		)\
	);" "seq"
}

#***********************************************************************
# mac_show_graphic_editor -- open the graphic editor for the current axes
#***********************************************************************
proc mac_show_graphic_editor {} {

	ScilabEval "if ~isempty(winsid()) then, ged(9), end;" "seq"
}

#***********************************************************************
# mac_edit_legend -- open a dialog box to edit legend texts in the current figure
#***********************************************************************
proc mac_edit_legend {} {

	ScilabEval "PLT_Macro_dboxEditLegend();" "seq"
}

#***********************************************************************
# mac_twinkle_curve -- open a dialog box to twinkle a curve in the current axes
#***********************************************************************
proc mac_twinkle_curve {} {

	ScilabEval "PLT_Macro_dboxTwinkleCurve()" "seq"
}

#***********************************************************************
# mac_delete_curve -- open a dialog box to delete a curve in the current axes
#***********************************************************************
proc mac_delete_curve {} {

	ScilabEval "PLT_Macro_dboxDeleteCurve();" "seq"
}

#***********************************************************************
# mac_export_curve_to_workspace -- open a dialog box to export curve data to workspace
#***********************************************************************
proc mac_export_curve_to_workspace {} {

	ScilabEval "\
		PLT_flg_rep = 1;\
		while PLT_flg_rep == 1 do PLT_flg_rep = PLT_Macro_dboxExportCurveToWS(); end;\
		clear PLT_flg_rep;\
	" "seq"
}

#***********************************************************************
# mac_exit_program -- clean up and exit program
#***********************************************************************
proc mac_exit_program {} {

	ScilabEval "clearglobal PLT;" "seq"
	destroy .w_PLT
}

#***********************************************************************
# mac_exit_scilab -- exit Scilab
#***********************************************************************
proc mac_exit_scilab {} {

	mac_exit_program
	ScilabEval "exit;" "seq"
}

#***********************************************************************
# mac_filter_data -- filter 'data'
#***********************************************************************
proc mac_filter_data {} {

	# <In>
	global DataNames; # list of all data names
	global DataFilterYesNo; # flag: whether to filter data or not
	global DataFilterPattern; # pattern string for data filtering
	# <Out>
	global FilteredData; # filtered list of data names
	global FilteredDataIndices; # list of indices of filtered data
	
	# Filter the data
	if {$DataFilterYesNo == 1} {
		set FilteredDataIndices [filter_list_indices $DataNames $DataFilterPattern]
	} else {
		set FilteredDataIndices [filter_list_indices $DataNames ""]
	}
	set FilteredData {}
	foreach i $FilteredDataIndices {
		lappend FilteredData [lindex $DataNames $i]
	}
}

#***********************************************************************
# mac_get_SelectedDataIndices -- get "SelectedDataIndices" from the data list-box
#***********************************************************************
proc mac_get_SelectedDataIndices {} {

	# <In>
	global FilteredDataIndices; # list of indices of filtered data
	# <Out>
	global SelectedDataIndices; # list of indices of currently selected data
	
	# Get indices of the selected data from the data list-box
	set SelectedDataIndices ""
	foreach i [$::frameData.lsb_Data curselection] {
		lappend SelectedDataIndices [lindex $FilteredDataIndices $i]
	}
}

#***********************************************************************
# mac_filter_SelectedDataIndices -- filter "SelectedDataIndices"
#***********************************************************************
proc mac_filter_SelectedDataIndices {} {

	# <In>
	global FilteredDataIndices; # list of indices of filtered data
	# <In/Out>
	global SelectedDataIndices; # list of indices of currently selected data
	
	# The new selected data is the intersection of the selected data and the filtered data.
	set SelectedDataIndices [common_elements [list $FilteredDataIndices $SelectedDataIndices]]
}

#***********************************************************************
# mac_decolor_data_listbox -- decolor all items in the data list-box
#***********************************************************************
proc mac_decolor_data_listbox {} {

	global FilteredDataIndices; # list of indices of filtered data
	
	for {set i 0} {$i < [llength $FilteredDataIndices]} {incr i} {
		$::frameData.lsb_Data itemconfigure $i -background white -foreground black
	}
}

#***********************************************************************
# mac_paint_data_listbox -- paint the selected items in the data list-box
#***********************************************************************
proc mac_paint_data_listbox {} {

	global FilteredDataIndices; # list of indices of filtered data
	global SelectedDataIndices; # list of indices of currently selected data
	
	for {set i 0} {$i < [llength $FilteredDataIndices]} {incr i} {
		if {[is_included [lindex $FilteredDataIndices $i] $SelectedDataIndices] == 1} {
			$::frameData.lsb_Data itemconfigure $i -background blue -foreground white
		} else {
			$::frameData.lsb_Data itemconfigure $i -background white -foreground black
		}
	}
}

#***********************************************************************
# mac_update_XVar -- update "FilteredXVars"
#***********************************************************************
proc mac_update_XVar {} {

	# <In>
	global VarNamesList; # list of lists of variable names
	global SelectedDataIndices; # list of indices of currently selected data
	global XVarFilterYesNo; # flag: whether to filter x variables or not
	global XVarFilterPattern; # pattern string for x variable filtering
	# <Out>
	global FilteredXVars; # filtered list of x variables
	
	# VarNames_selected is a subset of VarNamesList whose elements are variable names of selected data.
	# Say VarNamesList = { {variable names of data_1} {variable names of data_2} {variable names of data_3} }
	# and selected data are data_1 and data_3, then
	# VarNames_selected = { {variable names of data_1} {variable names of data_3} }.
	set VarNames_selected ""
	foreach i $SelectedDataIndices {
		lappend VarNames_selected [lindex $VarNamesList $i]
	}
	set commonVarNames [common_elements $VarNames_selected]
	
	# Filter the variables
	if {$XVarFilterYesNo == 1} {
		set FilteredXVars [filter_list $commonVarNames $XVarFilterPattern]
	} else {
		set FilteredXVars [filter_list $commonVarNames ""]
	}
}

#***********************************************************************
# mac_get_SelectedXVars -- get "SelectedXVars" from the x variable list-box
#***********************************************************************
proc mac_get_SelectedXVars {} {

	# <In>
	global FilteredXVars; # filtered list of x variables
	# <Out>
	global SelectedXVars; # list of currently selected x variables

	set SelectedXVars ""
	foreach i [$::frameXVar.lsb_XVar curselection] {
		lappend SelectedXVars [lindex $FilteredXVars $i]
	}
}

#***********************************************************************
# mac_filter_SelectedXVars -- filter "SelectedXVars"
#***********************************************************************
proc mac_filter_SelectedXVars {} {

	# <In>
	global FilteredXVars; # filtered list of x variables
	# <In/Out>
	global SelectedXVars; # list of currently selected x variables

	# The new selected variables are the intersection of the selected variables and the filtered variables
	set SelectedXVars [common_elements [list $FilteredXVars $SelectedXVars]]
}

#***********************************************************************
# mac_update_YVar -- update "FilteredYVars"
#***********************************************************************
proc mac_update_YVar {} {

	# <In>
	global VarNamesList; # list of lists of variable names
	global SelectedDataIndices; # list of indices of currently selected data
	global YVarFilterYesNo; # flag: whether to filter y variables or not
	global YVarFilterPattern; # pattern string for y variable filtering
	# <Out>
	global FilteredYVars; # filtered list of y variables
	
	# VarNames_selected is a subset of VarNamesList whose elements are variable names of selected data.
	# Say VarNamesList = { {variable names of data_1} {variable names of data_2} {variable names of data_3} }
	# and selected data are data_1 and data_3, then
	# VarNames_selected = { {variable names of data_1} {variable names of data_3} }.
	set VarNames_selected ""
	foreach index $SelectedDataIndices {
		lappend VarNames_selected [lindex $VarNamesList $index]
	}
	set commonVarNames [common_elements $VarNames_selected]
	
	# Filter the variables
	if {$YVarFilterYesNo == 1} {
		set FilteredYVars [filter_list $commonVarNames $YVarFilterPattern]
	} else {
		set FilteredYVars [filter_list $commonVarNames ""]
	}
}

#***********************************************************************
# mac_get_SelectedYVars -- get "SelectedYVars" from the y variable list-box
#***********************************************************************
proc mac_get_SelectedYVars {} {

	# <In>
	global FilteredYVars; # filtered list of y variables
	# <Out>
	global SelectedYVars; # list of currently selected y variables

	set SelectedYVars ""
	foreach i [$::frameYVar.lsb_YVar curselection] {
		lappend SelectedYVars [lindex $FilteredYVars $i]
	}
}

#***********************************************************************
# mac_filter_SelectedYVars -- filter "SelectedYVars"
#***********************************************************************
proc mac_filter_SelectedYVars {} {

	# <In>
	global FilteredYVars; # filtered list of y variables
	# <In/Out>
	global SelectedYVars; # list of currently selected y variables

	# The new selected variables are the intersection of the selected variables and the filtered variables
	set SelectedYVars [common_elements [list $FilteredYVars $SelectedYVars]]
}

#***********************************************************************
# mac_decolor_xvar_listbox -- decolor all items in the x variable list-box
#***********************************************************************
proc mac_decolor_xvar_listbox {} {

	global FilteredXVars; # filtered list of x variables
	
	for {set i 0} {$i < [llength $FilteredXVars]} {incr i} {
		$::frameXVar.lsb_XVar itemconfigure $i -background white -foreground black
	}
}

#***********************************************************************
# mac_paint_xvar_listbox -- paint the selected items in the x variable list-box
#***********************************************************************
proc mac_paint_xvar_listbox {} {

	global FilteredXVars; # filtered list of x variables
	global SelectedXVars; # list of currently selected x variables
	
	for {set i 0} {$i < [llength $FilteredXVars]} {incr i} {
		if {[is_included [lindex $FilteredXVars $i] $SelectedXVars] == 1} {
			$::frameXVar.lsb_XVar itemconfigure $i -background blue -foreground white
		} else {
			$::frameXVar.lsb_XVar itemconfigure $i -background white -foreground black
		}
	}
}

#***********************************************************************
# mac_decolor_yvar_listbox -- decolor all items in the y variable list-box
#***********************************************************************
proc mac_decolor_yvar_listbox {} {

	global FilteredYVars; # filtered list of y variables
	
	for {set i 0} {$i < [llength $FilteredYVars]} {incr i} {
		$::frameYVar.lsb_YVar itemconfigure $i -background white -foreground black
	}
}

#***********************************************************************
# mac_paint_yvar_listbox -- paint the selected items in the y variable list-box
#***********************************************************************
proc mac_paint_yvar_listbox {} {

	global FilteredYVars; # filtered list of y variables
	global SelectedYVars; # list of currently selected y variables
		
	for {set i 0} {$i < [llength $FilteredYVars]} {incr i} {
		if {[is_included [lindex $FilteredYVars $i] $SelectedYVars] == 1} {
			$::frameYVar.lsb_YVar itemconfigure $i -background blue -foreground white
		} else {
			$::frameYVar.lsb_YVar itemconfigure $i -background white -foreground black
		}
	}
}

#***********************************************************************
# mac_import_datafilter -- import preset data filter
#***********************************************************************
proc mac_import_datafilter {} {

	global CurrentDir; # current directory path
	global PresetDataFilterPatterns; # preset data filter patterns

	set typelist {
		{"All Files" {*}}
		{"Text Files" {".txt"}}
	}
	set filePath [tk_getOpenFile -initialdir $CurrentDir -parent .w_PLT -filetypes $typelist]
	if {$filePath ne ""} {
		if {[file exists $filePath] != 1} {
			tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
				-message "File \"$filePath\" doesn't exist or isn't accessible."
			return
		}
		set lines [read_file $filePath]
		set PresetDataFilterPatterns ""
		foreach line $lines {
			lappend PresetDataFilterPatterns $line
		}
		$::frameData.cmb_DataFilter configure -values $PresetDataFilterPatterns
	}
}

#***********************************************************************
# mac_import_varfilter -- import preset variable filter
#***********************************************************************
proc mac_import_varfilter {} {

	global CurrentDir; # current directory path
	global PresetVarFilterPatterns; # preset variable filter patterns

	set typelist {
		{"All Files" {*}}
		{"Text Files" {".txt"}}
	}
	set filePath [tk_getOpenFile -initialdir $CurrentDir -parent .w_PLT -filetypes $typelist]
	if {$filePath ne ""} {
		if {[file exists $filePath] != 1} {
			tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!"\
				-message "File \"$filePath\" doesn't exist or isn't accessible."
			return
		}
		set lines [read_file $filePath]
		set PresetVarFilterPatterns ""
		foreach line $lines {
			lappend PresetVarFilterPatterns $line
		}
		$::frameXVar.cmb_XVarFilter configure -values $PresetVarFilterPatterns
		$::frameYVar.cmb_YVarFilter configure -values $PresetVarFilterPatterns
	}
}

#***********************************************************************
# mac_change_curdirectory -- change current directory
#***********************************************************************
proc mac_change_curdirectory {} {

	global CurrentDir; # current directory path
	
	set dir [tk_chooseDirectory -initialdir $CurrentDir -parent .w_PLT -title "Choose a directory"]
	if {$dir ne ""} {
		set CurrentDir $dir
	}
}

#***********************************************************************
# mac_show_curdirectory -- show current directory
#***********************************************************************
proc mac_show_curdirectory {} {

	global CurrentDir; # current directory path
	
	tk_messageBox -type ok -icon info -parent .w_PLT -title "Current directory"\
		-message "$CurrentDir"
}

#***********************************************************************
# mac_about_software -- show information about this software
#***********************************************************************
proc mac_about_software {} {

	tk_messageBox -type ok -icon info -parent .w_PLT -title "About this software"\
		-message "$::SOFTWARE_NAME Version $::SOFTWARE_VERSION"\
		-detail "Copyright (c) $::COPYRIGHT\n\n$::LICENSE_NOTICE"
}

#***********************************************************************
# mac_open_manual -- open user's manual with external browser
#***********************************************************************
proc mac_open_manual {} {

	if {[catch {invoke_browser $::USERS_MANUAL_FILE} result]} {
		tk_messageBox -parent .w_PLT -type ok -icon error -title "Error!" -message $result
	}
}
	