# This file is part of QuickPlot

#***********************************************************************
# Main GUI Module
#
#     Construct the main GUI
#***********************************************************************

#package require BWidget
#package require img::png

# Global Constants
set SOFTWARE_NAME "QuickPlot"
set SOFTWARE_VERSION "2.0.2"
set COPYRIGHT "2019-2020 Sky Engineering Laboratory Inc."
set LICENSE_NOTICE "QuickPlot is free software: you can redistribute it and/or modify\
it under the terms of the GNU General Public License as published by\
the Free Software Foundation, either version 3 of the License, or\
(at your option) any later version.

QuickPlot is distributed in the hope that it will be useful,\
but WITHOUT ANY WARRANTY; without even the implied warranty of\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\
GNU General Public License for more details.

You should have received a copy of the GNU General Public License\
along with QuickPlot.  If not, see <https://www.gnu.org/licenses/>.
"
set OS [lindex $tcl_platform(os) 0]
set INSTALL_DIR [file dirname [info script]];
if {$OS == "Windows"} {
	set ICON_FILE [file join $INSTALL_DIR "quickplot.ico"];
} else {
	set ICON_FILE ""
}
set USERS_MANUAL_FILE [file join $INSTALL_DIR "../QuickPlot_UsersManual.pdf"]

# Global Variables
set CurrentDir $env(HOME); # current directory path (initially user's home directory)
set DataNames {}; # list of all data names
set VarNamesList {}; # list of lists of variable names
set FilteredData {}; # filtered list of data names
set FilteredXVars {}; # filtered list of x variables
set FilteredYVars {}; # filtered list of y variables
set SelectedDataIndices {}; # list of indices of currently selected data
set SelectedXVars {}; # list of currently selected x variables
set SelectedYVars {}; # list of currently selected y variables
set PresetDataFilterPatterns {}; # preset data filter patterns 
set PresetVarFilterPatterns {}; # preset variable filter patterns 
set DataFilterPattern {}; # pattern string for data filtering
set XVarFilterPattern {}; # pattern string for x variable filtering
set YVarFilterPattern {}; # pattern string for y variable filtering
set FilteredDataIndices {}; # list of indices of filtered data
set XVarYesNo 0; # flag: whether to use x variable or not
set DataFilterYesNo 0; # flag: whether to filter data or not
set XVarFilterYesNo 0; # flag: whether to filter x variables or not
set YVarFilterYesNo 0; # flag: whether to filter y variables or not
set PlotTitle ""; # plot title string
set PlotXLabel ""; # plot x label string
set PlotYLabel ""; # plot y label string
set PlotFontSize 1; # plot font size
set PlotLineYesNo 1; # flag: whether to show plot lines or not
set PlotLineThickness 1.0; # plot line thickness
set PlotMarkYesNo 0; # flag: whether to show plot marks or not
set PlotMarkSize 4.0; # mark size
set PlotGridYesNo 0; # flag: whether to show plot grids or not
set PlotGridColor 31; # plot grid color
set PlotLegendYesNo 1; # flag: whether to show plot legend or not
set PlotLegendFontSize 1; # plot legend font size
set PlotLegendLocation "in_upper_right"; # plot legend location
set PlotXLimYesNo 0; # flag: whether to use plot x limits or not
set PlotYLimYesNo 0; # flag: whether to use plot y limits or not
set PlotXMin 0; # plot lower limit of x
set PlotXMax 1; # plot upper limit of x
set PlotYMin 0; # plot lower limit of y
set PlotYMax 1; # plot upper limit of y
set BrowseYesNo 0; # flag: whether if browse mode is on or not
set ApplyPropAlwaysYesNo 0; # flag: whether if the graph property is applied always or not
set PropertyIsShown 0; # flag: whether if Graph Property GUI is shown or not
set CurrentFigID {}; # current figure ID
set ListFigIDs {}; # list of figure IDs

# Read separate source files
source [file join $INSTALL_DIR "procs_Callbacks.tcl"]
source [file join $INSTALL_DIR "procs_Functions.tcl"]
source [file join $INSTALL_DIR "procs_Macros.tcl"]
source [file join $INSTALL_DIR "dbox_ImportData.tcl"]
source [file join $INSTALL_DIR "dbox_ImportVarNames.tcl"]
source [file join $INSTALL_DIR "dbox_RenameData.tcl"]
source [file join $INSTALL_DIR "dbox_PermuteData.tcl"]
source [file join $INSTALL_DIR "dbox_ShowData.tcl"]
source [file join $INSTALL_DIR "dbox_EditVarNames.tcl"]
source [file join $INSTALL_DIR "dbox_CreateVar.tcl"]
source [file join $INSTALL_DIR "dbox_DeleteVar.tcl"]
source [file join $INSTALL_DIR "dbox_DeleteData.tcl"]
source [file join $INSTALL_DIR "dbox_EditPresetFilters.tcl"]
source [file join $INSTALL_DIR "pop_Data.tcl"]
source [file join $INSTALL_DIR "pop_Variable.tcl"]
source [file join $INSTALL_DIR "dbox_InsertVarval.tcl"]

# For Tcl/Tk debugging, can ignore all ScilabEvals with the following:
# proc ScilabEval {args} {
# }

#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
# Main Window
#_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
toplevel .w_PLT

# Window Manager
wm title .w_PLT "$SOFTWARE_NAME"
wm protocol .w_PLT WM_DELETE_WINDOW mac_exit_program
wm iconbitmap .w_PLT $::ICON_FILE
#wm geometry .w_PLT "680x290"

# Menu Bar
menu .w_PLT.menubar
.w_PLT config -menu .w_PLT.menubar
bind .w_PLT <ButtonPress-1> {mac_get_current_fig_id}

set mFile [menu .w_PLT.menubar.mFile -tearoff false]
.w_PLT.menubar add cascade -label File -menu .w_PLT.menubar.mFile

set mEdit [menu .w_PLT.menubar.mEdit -tearoff false]
.w_PLT.menubar add cascade -label Edit -menu .w_PLT.menubar.mEdit

set mCurve [menu .w_PLT.menubar.mCurve -tearoff false]
.w_PLT.menubar add cascade -label Curve -menu .w_PLT.menubar.mCurve

set mHelp [menu .w_PLT.menubar.mHelp -tearoff false]
.w_PLT.menubar add cascade -label Help -menu .w_PLT.menubar.mHelp

$mFile add command -label "Import Data" -command dbox_ImportData
$mFile add command -label "Import Variable Names" -command dbox_ImportVarNames
$mFile add separator
$mFile add command -label "Clear Data" -command mac_clear
$mFile add separator
$mFile add command -label "Import Data Filter" -command mac_import_datafilter
$mFile add command -label "Import Variable Filter" -command mac_import_varfilter
$mFile add separator
$mFile add command -label "Change Current Directory" -command mac_change_curdirectory
$mFile add command -label "Show Current Directory" -command mac_show_curdirectory
$mFile add separator
$mFile add command -label "Quit QuickPlot" -command mac_exit_program
$mFile add command -label "Quit Scilab" -command mac_exit_scilab

$mEdit add command -label "Rename Data" -command dbox_RenameData
$mEdit add command -label "Permute Data" -command dbox_PermuteData
$mEdit add command -label "Show Data" -command dbox_ShowData
$mEdit add command -label "Delete Data" -command dbox_DeleteData
$mEdit add separator
$mEdit add command -label "Edit Variable Names" -command dbox_EditVarNames
$mEdit add command -label "Create Variable" -command dbox_CreateVar
$mEdit add command -label "Delete Variable" -command dbox_DeleteVar
$mEdit add separator
$mEdit add command -label "Edit Preset Filters" -command dbox_EditPresetFilters

$mCurve add command -label "Twinkle" -command mac_twinkle_curve
$mCurve add command -label "Delete" -command mac_delete_curve
$mCurve add command -label "Export To Workspace" -command mac_export_curve_to_workspace

$mHelp add command -label "User's Manual" -command mac_open_manual
$mHelp add command -label "About This Software" -command mac_about_software

set pmenuData [menu .w_PLT.popupMenuData -tearoff false]
$pmenuData add command -label "Rename Data" -command dbox_RenameData
$pmenuData add command -label "Permute Data" -command dbox_PermuteData
$pmenuData add command -label "Show Data" -command pop_ShowData
$pmenuData add command -label "Delete Data" -command pop_DeleteData
$pmenuData add separator
$pmenuData add command -label "Edit Variable Names" -command dbox_EditVarNames

set pmenuXVar [menu .w_PLT.popupMenuXVar -tearoff false]
$pmenuXVar add command -label "Edit Variable Names" -command dbox_EditVarNames
$pmenuXVar add command -label "Create Variable" -command dbox_CreateVar
$pmenuXVar add command -label "Delete Variable" -command {pop_DeleteVariable "x"}

set pmenuYVar [menu .w_PLT.popupMenuYVar -tearoff false]
$pmenuYVar add command -label "Edit Variable Names" -command dbox_EditVarNames
$pmenuYVar add command -label "Create Variable" -command dbox_CreateVar
$pmenuYVar add command -label "Delete Variable" -command {pop_DeleteVariable "y"}

# In the main window, create "Upper" and "Lower" frames
set frameUpper [frame .w_PLT.f_Upper]
set frameLower [frame .w_PLT.f_Lower]
grid $frameUpper -sticky news
grid $frameLower -sticky news
grid rowconfigure .w_PLT 0 -weight 1
grid columnconfigure .w_PLT 0 -weight 1

# In Upper frame, create "Plot" frame and "ShowHideProperty" button
set framePlot [frame $frameUpper.f_Plot]
set buttonShowHideProperty [button $frameUpper.btn_ShowHideProperty\
	-text {Show Property Setting >>} -bg {dimgray} -fg white -command {cb_btn_ShowHideProperty}]
grid $framePlot -padx 10 -pady 10 -sticky news
grid $buttonShowHideProperty -padx 10 -pady 6 -sticky w
grid rowconfigure $frameUpper 0 -weight 1;# -minsize 300
grid columnconfigure $frameUpper 0 -weight 1

# In Lower frame, create "Property" frame
set frameProperty [labelframe $frameLower.lf_Property -text "Graph Property"]
grid $frameProperty -padx 10 -pady 10 -sticky news
grid columnconfigure $frameLower 0 -weight 1

# In Plot frame, create "DataArea" paned-window and "PlotButtons" frame side-by-side.
set paneDataArea [panedwindow $framePlot.pw_DataArea]
set framePlotButtons [frame $framePlot.f_Buttons]
grid $paneDataArea $framePlotButtons -sticky news
grid rowconfigure $framePlot 0 -weight 1
grid columnconfigure $framePlot 0 -weight 1

# In DataArea paned-window, create three frames in a row: "Data", "XVar", and "YVar"
set frameData [labelframe $paneDataArea.lf_Data -text "Data"]
checkbutton $paneDataArea.ckb_XVar -text "X variable" -variable XVarYesNo -anchor w -command cb_ckb_XVar
set frameXVar [labelframe $paneDataArea.lf_XVar -labelwidget $paneDataArea.ckb_XVar]
set frameYVar [labelframe $paneDataArea.lf_YVar -text "Y variable"]
$paneDataArea add $frameData -stretch always
$paneDataArea add $frameXVar -stretch always
$paneDataArea add $frameYVar -stretch always

# In Data frame:
listbox $frameData.lsb_Data -height 10 -xscrollcommand {$frameData.scrl_Data_h set} -yscrollcommand {$frameData.scrl_Data_v set}\
	-selectmode extended -selectbackground blue -selectforeground white -listvariable FilteredData -activestyle none
bind $frameData.lsb_Data <ButtonPress-1> {cb_select_data_press}
bind $frameData.lsb_Data <KeyPress-Up> {cb_select_data_press}
bind $frameData.lsb_Data <KeyPress-Down> {cb_select_data_press}
bind $frameData.lsb_Data <ButtonRelease-1> {cb_select_data_release}
bind $frameData.lsb_Data <KeyRelease-Up> {cb_select_data_release}
bind $frameData.lsb_Data <KeyRelease-Down> {cb_select_data_release}
bind $frameData.lsb_Data <ButtonPress-3> {if {$SelectedDataIndices != ""} {tk_popup .w_PLT.popupMenuData %X %Y}}
scrollbar $frameData.scrl_Data_h -orient horizontal -command {$frameData.lsb_Data xview}
scrollbar $frameData.scrl_Data_v -orient vertical -command {$frameData.lsb_Data yview}
checkbutton $frameData.ckb_DataFilter -text "Filter" -variable DataFilterYesNo -anchor w -command cb_ckb_DataFilter
ttk::combobox $frameData.cmb_DataFilter -textvariable DataFilterPattern -state disabled -values $PresetDataFilterPatterns
bind $frameData.cmb_DataFilter <Return> {cb_filter_data; cb_ckb_Browse}
bind $frameData.cmb_DataFilter <<ComboboxSelected>> {cb_filter_data; cb_ckb_Browse}
grid $frameData.lsb_Data $frameData.scrl_Data_v -sticky news
grid $frameData.scrl_Data_h -sticky news
grid $frameData.ckb_DataFilter -sticky ew
grid $frameData.cmb_DataFilter -sticky ew
grid rowconfigure $frameData 0 -weight 1
grid columnconfigure $frameData 0 -weight 1
$frameData.lsb_Data configure\
	-xscrollcommand [list sbset $frameData.scrl_Data_h]\
	-yscrollcommand [list sbset $frameData.scrl_Data_v]

# In XVar frame:
listbox $frameXVar.lsb_XVar -height 10 -xscrollcommand {$frameXVar.scrl_XVar_h set} -yscrollcommand {$frameXVar.scrl_XVar_v set}\
	-selectmode browse -selectbackground blue -selectforeground white -listvariable FilteredXVars -activestyle none -state disabled
bind $frameXVar.lsb_XVar <ButtonPress-1> {cb_select_xvar_press}
bind $frameXVar.lsb_XVar <KeyPress-Up> {cb_select_xvar_press}
bind $frameXVar.lsb_XVar <KeyPress-Down> {cb_select_xvar_press}
bind $frameXVar.lsb_XVar <ButtonRelease-1> {cb_select_xvar_release}
bind $frameXVar.lsb_XVar <KeyRelease-Up> {cb_select_xvar_release}
bind $frameXVar.lsb_XVar <KeyRelease-Down> {cb_select_xvar_release}
bind $frameXVar.lsb_XVar <ButtonPress-3> {if {$SelectedXVars != "" && $XVarYesNo == 1} {tk_popup .w_PLT.popupMenuXVar %X %Y}}
scrollbar $frameXVar.scrl_XVar_h -orient horizontal -command {$frameXVar.lsb_XVar xview}
scrollbar $frameXVar.scrl_XVar_v -orient vertical -command {$frameXVar.lsb_XVar yview}
checkbutton $frameXVar.ckb_XVarFilter -text "Filter" -variable XVarFilterYesNo\
	-anchor w -command cb_ckb_XVarFilter -state disabled
ttk::combobox $frameXVar.cmb_XVarFilter -textvariable XVarFilterPattern -state disabled -values $PresetVarFilterPatterns
bind $frameXVar.cmb_XVarFilter <Return> {cb_filter_xvar; cb_ckb_Browse}
bind $frameXVar.cmb_XVarFilter <<ComboboxSelected>> {cb_filter_xvar; cb_ckb_Browse}
grid $frameXVar.lsb_XVar $frameXVar.scrl_XVar_v -sticky news
grid $frameXVar.scrl_XVar_h -sticky news
grid $frameXVar.ckb_XVarFilter -sticky ew
grid $frameXVar.cmb_XVarFilter -sticky ew
grid rowconfigure $frameXVar 0 -weight 1
grid columnconfigure $frameXVar 0 -weight 1
$frameXVar.lsb_XVar configure\
	-xscrollcommand [list sbset $frameXVar.scrl_XVar_h]\
	-yscrollcommand [list sbset $frameXVar.scrl_XVar_v]

# In YVar frame:
listbox $frameYVar.lsb_YVar -height 10 -xscrollcommand {$frameYVar.scrl_YVar_h set} -yscrollcommand {$frameYVar.scrl_YVar_v set}\
	-selectmode extended -selectbackground blue -selectforeground white -listvariable FilteredYVars -activestyle none
bind $frameYVar.lsb_YVar <ButtonPress-1> {cb_select_yvar_press}
bind $frameYVar.lsb_YVar <KeyPress-Up> {cb_select_yvar_press}
bind $frameYVar.lsb_YVar <KeyPress-Down> {cb_select_yvar_press}
bind $frameYVar.lsb_YVar <ButtonRelease-1> {cb_select_yvar_release}
bind $frameYVar.lsb_YVar <KeyRelease-Up> {cb_select_yvar_release}
bind $frameYVar.lsb_YVar <KeyRelease-Down> {cb_select_yvar_release}
bind $frameYVar.lsb_YVar <ButtonPress-3> {if {$SelectedYVars != ""} {tk_popup .w_PLT.popupMenuYVar %X %Y}}
scrollbar $frameYVar.scrl_YVar_h -orient horizontal -command {$frameYVar.lsb_YVar xview}
scrollbar $frameYVar.scrl_YVar_v -orient vertical -command {$frameYVar.lsb_YVar yview}
checkbutton $frameYVar.ckb_YVarFilter -text "Filter" -variable YVarFilterYesNo -anchor w -command cb_ckb_YVarFilter
ttk::combobox $frameYVar.cmb_YVarFilter -textvariable YVarFilterPattern -state disabled -values $PresetVarFilterPatterns
bind $frameYVar.cmb_YVarFilter <Return> {cb_filter_yvar; cb_ckb_Browse}
bind $frameYVar.cmb_YVarFilter <<ComboboxSelected>> {cb_filter_yvar; cb_ckb_Browse}
grid $frameYVar.lsb_YVar $frameYVar.scrl_YVar_v -sticky news
grid $frameYVar.scrl_YVar_h -sticky news
grid $frameYVar.ckb_YVarFilter -sticky ew
grid $frameYVar.cmb_YVarFilter -sticky ew
grid rowconfigure $frameYVar 0 -weight 1
grid columnconfigure $frameYVar 0 -weight 1
$frameYVar.lsb_YVar configure\
	-xscrollcommand [list sbset $frameYVar.scrl_YVar_h]\
	-yscrollcommand [list sbset $frameYVar.scrl_YVar_v]

# In PlotButtons frame:
label $framePlotButtons.lab_CurrentFigID -text {Current Figure:} -anchor w;
ttk::combobox $framePlotButtons.cmb_CurrentFigID -textvariable CurrentFigID -state readonly -values $ListFigIDs -width 6
bind $framePlotButtons.cmb_CurrentFigID <<ComboboxSelected>> cb_cmb_CurrentFigID
mac_get_current_fig_id
button $framePlotButtons.btn_PlotAdd -text "Add Curves" -bg firebrick3 -fg white -command {mac_plot_data 0} -font {Aerial 10 bold}
button $framePlotButtons.btn_Clear -text "Clear Figure" -command {mac_clear_figure}
button $framePlotButtons.btn_NewFig -text "New Figure" -command {mac_new_figure}
button $framePlotButtons.btn_CloseAll -text "Close All" -command {mac_close_all}
checkbutton $framePlotButtons.ckb_Browse -text "Browse" -variable BrowseYesNo -anchor w -command cb_ckb_Browse
grid $framePlotButtons.lab_CurrentFigID -padx 10 -pady 2 -sticky w
grid $framePlotButtons.cmb_CurrentFigID -padx 10 -pady 2 -sticky e
grid $framePlotButtons.btn_PlotAdd -padx 10 -pady 2 -ipadx 10 -ipady 4 -sticky ew
grid $framePlotButtons.btn_Clear -padx 10 -pady 2 -ipadx 10 -ipady 4 -sticky ew
grid $framePlotButtons.btn_NewFig -padx 10 -pady 2 -ipadx 10 -ipady 4 -sticky ew
grid $framePlotButtons.btn_CloseAll -padx 10 -pady 2 -ipadx 10 -ipady 4 -sticky ew
grid $framePlotButtons.ckb_Browse -padx 10 -pady 2 -sticky ew

# In "Property" frame, create three frames in a row: "Column0", "Column1", and "PropButtons"
frame $frameProperty.f_Column0
frame $frameProperty.f_Column1
set framePropButtons [frame $frameProperty.f_Buttons]
grid $frameProperty.f_Column0 $frameProperty.f_Column1 $framePropButtons -padx 4 -pady 4 -sticky news
grid columnconfigure $frameProperty 0 -weight 1

# In Column0 frame, create "TitleAndAxisLabels" frame, "XLim" frame, and "YLim" frame
set frameTitleAndAxisLabels [labelframe $frameProperty.f_Column0.lf_TitleAndAxisLabels -text "Title & Axis Labels"]
checkbutton $frameProperty.f_Column0.ckb_XLim -text "X limits" -variable PlotXLimYesNo -anchor w -command cb_ckb_XLim
set frameXLim [labelframe $frameProperty.f_Column0.lf_XLim -labelwidget $frameProperty.f_Column0.ckb_XLim]
checkbutton $frameProperty.f_Column0.ckb_YLim -text "Y limits" -variable PlotYLimYesNo -anchor w -command cb_ckb_YLim
set frameYLim [labelframe $frameProperty.f_Column0.lf_YLim -labelwidget $frameProperty.f_Column0.ckb_YLim]
grid $frameTitleAndAxisLabels -padx 2 -pady 2 -sticky news
grid $frameXLim -padx 2 -pady 2 -sticky news
grid $frameYLim -padx 2 -pady 2 -sticky news
grid columnconfigure $frameProperty.f_Column0 0 -weight 1

# In TitleAndAxisLabels frame:
label $frameTitleAndAxisLabels.lab_Title -text "Title" -anchor w
entry $frameTitleAndAxisLabels.ent_Title -textvariable PlotTitle
label $frameTitleAndAxisLabels.lab_XLabel -text "X label" -anchor w
entry $frameTitleAndAxisLabels.ent_XLabel -textvariable PlotXLabel
label $frameTitleAndAxisLabels.lab_YLabel -text "Y label" -anchor w
entry $frameTitleAndAxisLabels.ent_YLabel -textvariable PlotYLabel
set PlotFontSizeList {0 1 2 3 4 5 6}
label $frameTitleAndAxisLabels.lab_FontSize -text "Font size" -anchor w
ttk::combobox $frameTitleAndAxisLabels.cmb_FontSize -textvariable PlotFontSize -state readonly -values $PlotFontSizeList -width 10
grid $frameTitleAndAxisLabels.lab_Title $frameTitleAndAxisLabels.ent_Title -padx 4 -pady 2 -sticky ew
grid $frameTitleAndAxisLabels.lab_XLabel $frameTitleAndAxisLabels.ent_XLabel -padx 4 -pady 2 -sticky ew
grid $frameTitleAndAxisLabels.lab_YLabel $frameTitleAndAxisLabels.ent_YLabel -padx 4 -pady 2 -sticky ew
grid $frameTitleAndAxisLabels.lab_FontSize $frameTitleAndAxisLabels.cmb_FontSize -padx 4 -pady 2 -sticky w
grid columnconfigure $frameTitleAndAxisLabels 1 -weight 1

# In XLim frame:
label $frameXLim.lab_XMin -text "Min" -anchor w -state disabled
entry $frameXLim.ent_XMin -textvariable PlotXMin -state disabled
label $frameXLim.lab_XMax -text "Max" -anchor w -state disabled
entry $frameXLim.ent_XMax -textvariable PlotXMax -state disabled
grid $frameXLim.lab_XMin $frameXLim.ent_XMin $frameXLim.lab_XMax $frameXLim.ent_XMax -padx 4 -pady 2 -sticky ew
grid columnconfigure $frameXLim {1 3} -weight 1

# In YLim frame:
label $frameYLim.lab_YMin -text "Min" -anchor w -state disabled
entry $frameYLim.ent_YMin -textvariable PlotYMin -state disabled
label $frameYLim.lab_YMax -text "Max" -anchor w -state disabled
entry $frameYLim.ent_YMax -textvariable PlotYMax -state disabled
grid $frameYLim.lab_YMin $frameYLim.ent_YMin $frameYLim.lab_YMax $frameYLim.ent_YMax -padx 4 -pady 2 -sticky ew
grid columnconfigure $frameYLim {1 3} -weight 1

# In Column1 frame, create "Polyline" frame, "Grid" frame, and "Legend" frame
set framePolyline [labelframe $frameProperty.f_Column1.lf_Polyline -text "Polyline"]
checkbutton $frameProperty.f_Column1.ckb_Grid -text "Grid" -variable PlotGridYesNo -anchor w -command cb_ckb_Grid
set frameGrid [labelframe $frameProperty.f_Column1.lf_Grid -labelwidget $frameProperty.f_Column1.ckb_Grid]
checkbutton $frameProperty.f_Column1.ckb_Legend -text "Legend" -variable PlotLegendYesNo -anchor w -command cb_ckb_Legend
set frameLegend [labelframe $frameProperty.f_Column1.lf_Legend -labelwidget $frameProperty.f_Column1.ckb_Legend]
grid $framePolyline -padx 2 -pady 2 -sticky news
grid $frameGrid -padx 2 -pady 2 -sticky news
grid $frameLegend -padx 2 -pady 2 -sticky news

# In Polyline frame:
checkbutton $framePolyline.ckb_Line -text "Line" -variable PlotLineYesNo -anchor w -command cb_ckb_Line
set PlotLineThicknessList {0.0 1.0 2.0 3.0 4.0 5.0 6.0 8.0 10.0}
ttk::combobox $framePolyline.cmb_LineThickness -textvariable PlotLineThickness -state readonly -values $PlotLineThicknessList -width 10
checkbutton $framePolyline.ckb_Mark -text "Mark" -variable PlotMarkYesNo -anchor w -command cb_ckb_Mark
set PlotMarkSizeList {0.5 1.0 2.0 3.0 4.0 5.0 6.0 8.0 10.0 15.0 20.0 25.0 30.0}
ttk::combobox $framePolyline.cmb_MarkSize -textvariable PlotMarkSize -state readonly -values $PlotMarkSizeList -width 10 -state disabled
#grid $framePolyline.lab_LineThickness $framePolyline.cmb_LineThickness -padx 4 -pady 2 -sticky w
grid $framePolyline.ckb_Line $framePolyline.cmb_LineThickness -padx 4 -pady 2 -sticky w
grid $framePolyline.ckb_Mark $framePolyline.cmb_MarkSize -padx 4 -pady 2 -sticky w

# In Grid frame:
set PlotGridColorList {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33}
label $frameGrid.lab_GridColor -text "Grid color" -anchor w -state disabled
ttk::combobox $frameGrid.cmb_GridColor -textvariable PlotGridColor -values $PlotGridColorList -width 10 -state disabled
grid $frameGrid.lab_GridColor $frameGrid.cmb_GridColor -padx 4 -pady 2 -sticky w

# In Legend frame:
set PlotLegendFontSizeList {0 1 2 3 4 5 6}
label $frameLegend.lab_LegendFontSize -text "Font size" -anchor w -state normal
ttk::combobox $frameLegend.cmb_LegendFontSize -textvariable PlotLegendFontSize -state readonly -values $PlotLegendFontSizeList -width 10
set PlotLegendLocationList {
in_upper_right
in_upper_left
in_lower_right
in_lower_left
out_upper_right
out_upper_left
out_lower_right
out_lower_left
upper_caption
lower_caption
}
label $frameLegend.lab_LegendLocation -text "Location" -anchor w -state normal
ttk::combobox $frameLegend.cmb_LegendLocation -textvariable PlotLegendLocation -state readonly -values $PlotLegendLocationList -width 15
button $frameLegend.btn_EditLegend -text "Edit Legend" -command {mac_edit_legend} -width 15
grid $frameLegend.lab_LegendFontSize $frameLegend.cmb_LegendFontSize -padx 4 -pady 2 -sticky w
grid $frameLegend.lab_LegendLocation $frameLegend.cmb_LegendLocation -padx 4 -pady 2 -sticky w
grid $frameLegend.btn_EditLegend -columnspan 2 -padx 10 -pady 4 -sticky w

# In PropButtons frame:
button $framePropButtons.btn_ApplyProperty -text "Apply" -bg {dodger blue} -fg white -font {Aerial 10 bold} -command {mac_set_properties}
checkbutton $framePropButtons.ckb_ApplyPropAlways -text "Apply always" -variable ApplyPropAlwaysYesNo\
	-anchor w -command cb_ckb_ApplyPropAlways
button $framePropButtons.btn_ShowGraphicEditor -text "Graphic Editor" -command {mac_show_graphic_editor}
grid $framePropButtons.btn_ApplyProperty -padx 10 -pady 2 -ipady 4 -sticky ew
grid $framePropButtons.ckb_ApplyPropAlways -padx 10 -pady 2 -sticky ew
grid $framePropButtons.btn_ShowGraphicEditor -padx 10 -pady 2 -ipady 4 -sticky ew

# Hide Property frame
grid remove $frameLower
